/*
 * XMLWriter.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.io;


import org.jabberstudio.jso.StreamAttribute;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.StreamText;
import org.jabberstudio.jso.io.XMLExporter;
import org.jabberstudio.jso.util.BasicStreamObjectVisitor;
import org.jabberstudio.jso.util.Utilities;

import net.outer_planes.jso.BasicStreamContext;
import net.outer_planes.jso.NamespaceNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.6
 */
public class XMLWriter implements XMLExporter {
    //Inner-classes
    private class Visitor extends BasicStreamObjectVisitor {
        public String encodeTextValue(String in) {
            StringBuffer    out = null;
            int             idx;
            int prev = 0;
            int len = in.length();
            String temp = null;
            
            for (idx = 0; idx < in.length(); idx++) {
                char    ch = in.charAt(idx);

                switch(ch) {
                    case '&':   temp = "&amp;";    break;
                    case '<':   temp = "&lt;";     break;
                    default: break;
                }
                if (temp != null) {
                    if(out == null) out = new StringBuffer();  
                    out.append(in.substring(prev,idx));
                    out.append(temp);
                    prev = idx + 1;
                    temp = null;
                }

            }
            
	    if (prev == 0) {
		return in;
	    } else {
		return (out.append(in.substring(prev))).toString();
	    }
        }
        public String encodeAttributeValue(String in) {
            StringBuffer    out = null;
            int             idx;
            int prev = 0;
            int len = in.length();
            String temp = null;
            
            for (idx = 0; idx < len; idx++) {
                char    ch = in.charAt(idx);

                switch(ch) {
                    case '&':   temp = "&amp;";    break;
                    case '<':   temp = "&lt;";     break;
                    case '>':   temp = "&gt;";     break;
                    case '\'':  temp = "&apos;";   break;
                    case '\"':  temp = "&quot;";   break;
		    default:;
                }

                if (temp != null) {
                    if (out == null) out = new StringBuffer();  
                    out.append(in.substring(prev,idx));
                    out.append(temp);
                    prev = idx + 1;
                    temp = null;
                }
            }
            
	    if (prev == 0) {
		return in;
	    } else {
		return (out.append(in.substring(prev))).toString();
	    }
        }
        public String generateQName(StreamElement elem) {
            String          local = elem.getLocalName();
            String          prefix = elem.getNamespacePrefix();
            StringBuffer    qname = new StringBuffer();

            if (Utilities.isValidString(prefix))
                qname.append(prefix).append(":");
            qname.append(local);

            return qname.toString();
        }
        public String generateQName(StreamAttribute attr) {
            String          local = attr.getLocalName();
            String          prefix = attr.getNamespacePrefix();
            StringBuffer    qname = new StringBuffer();
            
            if (Utilities.isValidString(prefix))
                qname.append(prefix).append(":");
            qname.append(local);

            return qname.toString();
        }

        public void visit(StreamElement in) {

            _Buffer.append("<");
            boolean prefixValid = false;

            String          prefix = in.getNamespacePrefix();
            String          localName = in.getLocalName(); 
            
            if (Utilities.isValidString(prefix)) {
                _Buffer.append(prefix).append(":");
                prefixValid = true;
            }
                
            _Buffer.append(localName);
            

            StreamElement   parent = in.getParent();
            String          pURI = (parent != null) ? parent.getNamespaceURI() : null;
            String          nsURI = in.getNamespaceURI();

            //Add this element's xmlns if not present
            if ((nsURI != null) && (prefix == null) &&
		!Utilities.equateStrings(nsURI, pURI)) {
                visit(new NamespaceNode(in, "", nsURI));
            }

            //Process Namespaces and Attributes
            super.visitNamespaces(in);
            super.visitAttributes(in);
            
            //Process children, then terminate
            if (in.listNodes().isEmpty()) {
                _Buffer.append("/>");
            } else {
                _Buffer.append(">");
                super.visitChildren(in);
                _Buffer.append("</");
                if (prefixValid) {
                    _Buffer.append(prefix).append(":");
                }
                _Buffer.append(localName).append(">");
            }

        }
        
        public void visit(StreamNamespace in) {
            String      prefix = in.getPrefix();
            String      uri = encodeAttributeValue(in.getURI());
            
            if (Utilities.equateStrings(uri, Utilities.XML_NAMESPACE))
                return;
            
            _Buffer.append(" ");
            if (Utilities.isValidString(prefix)) {
                _Buffer.append("xmlns:").append(prefix);
            } else {
                _Buffer.append("xmlns");
            }
            _Buffer.append("=\'").append(uri).append("\'");
        }
        public void visit(StreamAttribute in) {
            String      qname = generateQName(in);
            String      value = encodeAttributeValue(in.getValue());
            

            _Buffer.append(" ").append(qname).append("=\'").append(value).append("\'");
        }
        
        public void visit(StreamText in) {
            String      out = encodeTextValue(in.getValue());
            _Buffer.append(out);
        }
    }
    
    //Fields
    private StreamElement   _Ctx;
    private StringBuffer    _Buffer;
    
    public XMLWriter() {
    }
    public XMLWriter(StreamElement ctx) throws IllegalArgumentException {
        configure(ctx);
    }
    
    //Methods
    public StreamElement getContext() {
        return _Ctx;
    }
    private void setContext(StreamElement ctx) throws IllegalArgumentException {
        if (ctx == null)
            ctx = new BasicStreamContext();
        _Ctx = ctx;
    }

    public synchronized String write(StreamElement in) throws RuntimeException {
        _Buffer = new StringBuffer();
        
        in.accept(new Visitor());
        
        return _Buffer.toString();
    }
    
    public void configure(StreamElement ctx) throws IllegalArgumentException, IllegalStateException, RuntimeException {
        if (getContext() != null)
            throw new IllegalStateException("This exporter is already configured");
        setContext(ctx);
    }
    public Object execute(StreamElement in) throws RuntimeException {
        return write(in);
    }
}
