/*
 * AbstractMechanismProvider.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl;


import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import org.jabberstudio.jso.sasl.SASLClientInfo;
import org.jabberstudio.jso.sasl.SASLMechanism;
import org.jabberstudio.jso.sasl.SASLMechanismManager;
import org.jabberstudio.jso.sasl.SASLMechanismProvider;
import org.jabberstudio.jso.sasl.SASLServerInfo;

/**
 *
 * @author  mmiller
 */
public abstract class AbstractMechanismProvider implements SASLMechanismProvider {
    //Inner-classes
    public static abstract class ClientEntry {
        //Fields
        private String  _Name;
        private Map     _Policy;
        
        //Constructors
        public ClientEntry(String name, Map policy) {
            _Name = name;
            _Policy = (policy != null) ? policy : Collections.EMPTY_MAP;
        }

        //Methods
        public final String getName() {
            return _Name;
        }
        public final Map getPolicy() {
            return _Policy;
        }
        
        public final boolean meetsPolicy(Map expect) {
            Map         actual = getPolicy();
            Iterator    itr = POLICY_KEYS.iterator();
            Object      key, val1, val2;
            Boolean     expectVal, actualVal;
            boolean     result = true;

            if (expect == null)
                expect = Collections.EMPTY_MAP;
            
            while (result && itr.hasNext()) {
                key = itr.next();
                val1 = expect.get(key);
                val2 = actual.get(key);

                expectVal = (val1 == null) ? Boolean.FALSE : Boolean.valueOf(val1.toString());
                actualVal = (val2 == null) ? Boolean.FALSE : Boolean.valueOf(val2.toString());
                
                result = !expectVal.booleanValue() ? true : (expectVal.booleanValue() && actualVal.booleanValue());
            }
            
            return result;
        }
        public abstract SASLMechanism createMechanism(SASLClientInfo info);
    }
    public static abstract class ServerEntry {
        //Fields
        private String  _Name;
        private Map     _Policy;
        
        //Constructors
        public ServerEntry(String name, Map policy) {
            _Name = name;
            _Policy = (policy != null) ? policy : Collections.EMPTY_MAP;
        }

        //Methods
        public final String getName() {
            return _Name;
        }
        public final Map getPolicy() {
            return _Policy;
        }
        
        public final boolean meetsPolicy(Map expect) {
            Map         actual = getPolicy();
            Iterator    itr = POLICY_KEYS.iterator();
            Object      key, val1, val2;
            Boolean     expectVal, actualVal;
            boolean     result = true;

            if (expect == null)
                expect = Collections.EMPTY_MAP;
            
            while (result && itr.hasNext()) {
                key = itr.next();
                val1 = expect.get(key);
                val2 = actual.get(key);

                expectVal = (val1 == null) ? Boolean.FALSE : Boolean.valueOf(val1.toString());
                actualVal = (val2 == null) ? Boolean.FALSE : Boolean.valueOf(val2.toString());
                
                result = !expectVal.booleanValue() ? true : (expectVal.booleanValue() && actualVal.booleanValue());
            }
            
            return result;
        }
        public abstract SASLMechanism createMechanism(SASLServerInfo info);
    }
    
    //"Constants"
    private static final List POLICY_KEYS = Arrays.asList(new String[] {
        SASLMechanismManager.PROPERTY_POLICY_NOANONYMOUS,
        SASLMechanismManager.PROPERTY_POLICY_NOACTIVE,
        SASLMechanismManager.PROPERTY_POLICY_NOPLAINTEXT,
        SASLMechanismManager.PROPERTY_POLICY_NODICTIONARY,
        SASLMechanismManager.PROPERTY_POLICY_FORWARD_SECRECY
    });
    
    //Fields
    private List    _ClientOrder = new java.util.ArrayList();
    private Map     _ClientMechs = new java.util.TreeMap();
    
    private List    _ServerOrder = new java.util.ArrayList();
    private Map     _ServerMechs = new java.util.TreeMap();
    
    //Constructors
    public AbstractMechanismProvider() {
        registerSupportedMechanisms();
    }
    
    //Methods
    protected Logger obtainLogger() {
        return Logger.getLogger(getClass());
    }
    
    protected abstract void registerSupportedMechanisms();
    protected void addSupportedClientMechanism(ClientEntry entry) {
        Map     mechs = getClientManchanisms();
        List    order = getClientOrdering();
        
        if ((entry != null) && !mechs.containsKey(entry.getName())) {
            order.add(entry.getName());
            mechs.put(entry.getName(), entry);
        }
    }
    protected void addSupportedServerMechanism(ServerEntry entry) {
        Map     mechs = getServerManchanisms();
        List    order = getServerOrdering();
        
        if ((entry != null) && !mechs.containsKey(entry.getName())) {
            order.add(entry.getName());
            mechs.put(entry.getName(), entry);
        }
    }

    protected List getClientOrdering() {
        return _ClientOrder;
    }
    protected Map getClientManchanisms() {
        return _ClientMechs;
    }
    public List getClientMechanismNames(Map props) {
        Map         mechs = getClientManchanisms();
        List        names = new java.util.ArrayList(getClientOrdering());
        Iterator    itr = names.iterator();
        String      name;
        ClientEntry entry;
        
        while (itr.hasNext()) {
            name = (String)itr.next();
            entry = (ClientEntry)mechs.get(name);
            
            if (!entry.meetsPolicy(props))
                itr.remove();
        }
        
        return names;
    }
    public SASLMechanism createClientMechanism(SASLClientInfo ci) throws IllegalArgumentException {
        Map                 mechs = getClientManchanisms();
        Map                 props = ci.getProperties();
        List                supported = ci.getMechanismNames();
        Iterator            itr = getClientOrdering().iterator();
        String              name;
        ClientEntry         entry;
        SASLMechanism       mech = null;

        while ((mech == null) && itr.hasNext()) {
            name = (String)itr.next();

            if (supported.contains(name)) {
                entry = (ClientEntry)mechs.get(name);
                
                if (entry.meetsPolicy(props))
                    mech = entry.createMechanism(ci);
            }
        }
        
        return mech;
    }
    
    protected Map getServerManchanisms() {
        return _ServerMechs;
    }
    protected List getServerOrdering() {
        return _ServerOrder;
    }
    public List getServerMechanismNames(Map props) {
        Map         mechs = getServerManchanisms();
        List        names = new java.util.ArrayList(getServerOrdering());
        Iterator    itr = names.iterator();
        String      name;
        ServerEntry entry;
        
        while (itr.hasNext()) {
            name = (String)itr.next();
            entry = (ServerEntry)mechs.get(name);
            
            if (!entry.meetsPolicy(props))
                itr.remove();
        }
        
        return names;
    }
    public SASLMechanism createServerMechanism(SASLServerInfo si) throws IllegalArgumentException {
        Map                 mechs = getServerManchanisms();
        Map                 props = si.getProperties();
        ServerEntry         entry = (ServerEntry)mechs.get(si.getMechanismName());
        SASLMechanism       mech = null;

        if ((entry != null) && entry.meetsPolicy(props))
            mech = entry.createMechanism(si);
        
        return mech;
    }
}