/*
 * ClientDigestMD5Mechanism.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl.mechanisms;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.SecureRandom;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.security.auth.callback.*;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.sasl.SASLAuthPacket;
import org.jabberstudio.jso.sasl.SASLClientInfo;
import org.jabberstudio.jso.sasl.SASLFailedException;
import org.jabberstudio.jso.sasl.SASLMechanism;
import org.jabberstudio.jso.sasl.SASLMechanismManager;
import org.jabberstudio.jso.sasl.SASLPacket;
import org.jabberstudio.jso.sasl.callback.*;
import org.jabberstudio.jso.util.DigestHash;
import org.jabberstudio.jso.util.ByteCodec;
import org.jabberstudio.jso.util.Utilities;

import net.outer_planes.jso.sasl.AbstractMechanismProvider;
import net.outer_planes.jso.sasl.ClientMechanism;

/**
 * <p>
 * Implementation of <tt>ClientMechanism</tt> to support DIGEST-MD5.</p>
 *
 * @author  hildjj
 * @version 0.13-DEV-20050624155226
 * @see     <a href='http://www.ietf.org/rfc/rfc2831.txt'>RFC 2831 -
 * Digest SASL Mechanism</a>
 * @since   JSO-0.12
 */
public class ClientDigestMD5Mechanism extends ClientMechanism {
    //Inner-classes
    public static class MD5Status extends SASLMechanism.Status {
        //Constructors
        private MD5Status(String name, boolean auth, boolean complete) {
            super(name, auth, complete);
        }
    }
    
    public static class Entry extends AbstractMechanismProvider.ClientEntry {
        //Constructors
        public Entry() {
            super(NAME, POLICY);
        }
        
        //Methods
        public SASLMechanism createMechanism(SASLClientInfo info) {
            return new ClientDigestMD5Mechanism(info);
        }
    }
    
    //"Constants"
    public static final Status  AUTHENTICATING_MORE = new MD5Status("final challenge", true, false);
    
    public static final String  NAME                = "DIGEST-MD5";
    public static final String  CB_AUTHZID          = "authzid";
    public static final String  CB_REALM            = "realm";
    public static final String  CB_USERNAME         = "username";
    public static final String  CB_PASSWORD         = "password";
    public static final String  CB_DOMAIN           = "domain";
    public static final String  CB_SERVER           = "server-name";
    
    public static final int     DEFAULT_MAXBUFFER   = 65536;
    
    public static final Map     POLICY;
    
    static {
        Map     temp = new java.util.TreeMap();
        
        temp.put(SASLMechanismManager.PROPERTY_POLICY_FORWARD_SECRECY, "false");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NOANONYMOUS, "true");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NOACTIVE, "true");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NODICTIONARY, "false");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_NOPLAINTEXT, "true");
        temp.put(SASLMechanismManager.PROPERTY_POLICY_PASS_CREDENTIALS, "false");
        
        POLICY = java.util.Collections.unmodifiableMap(temp);
    }
    
    //Fields
    private int     _ChallengeState      = 0;
    
    //Constructors
    public ClientDigestMD5Mechanism(SASLClientInfo ci) {
        super(ci);
    }
    
    //Methods
    public String getMechanismName() {
        return NAME;
    }
    
    private static final byte[] toBytes(String s) {
        byte[]  sb;
        
        if (s == null) {
            sb = new byte[0];
        } else {
            try {
                sb = s.getBytes("UTF-8");
            } catch (UnsupportedEncodingException e) {
                throw new IllegalStateException("UTF-8 is not a supported encoding");
            }
        }
        
        return sb;
    }
    
     /**
     * Let HEX(n) be the representation of the 16 octet MD5 hash n as
     * a string of 32 hex digits (with alphabetic characters always in
     * lower case, since MD5 is case sensitive).
     */
    private static final String toHex(byte[] hash) {
        return ByteCodec.HEX.encode(hash);
    }
    
   /**
     * <p>
     * Let H(s) be the 16 octet MD5 hash [RFC 1321] of the octet
     * string <tt>s</tt></p>.
     */
    private static final byte[] H(String s) {
        return H(toBytes(s));
    }
    
    private static final byte[] H(byte[] s) {
        try {
            MessageDigest   md = MessageDigest.getInstance("MD5");
            
            return md.digest(s);
        } catch (java.security.NoSuchAlgorithmException e) {
            throw new IllegalStateException("MD5 is not a supported hashing algorithm");
        }
    }
    
    private static final byte[] H(byte[] one, byte[] two) {
        try {
            MessageDigest   md = MessageDigest.getInstance("MD5");
            
            md.update(one);
            return md.digest(two);
        } catch (java.security.NoSuchAlgorithmException e) {
            throw new AssertionError("MD5 not found");
        }
    }
    
    /**
     * Let KD(k, s) be H({k, ":", s}), i.e., the 16 octet hash of the
     * string k, a colon and the string s.
     */
    private static final byte[] KD(String k, String s) {
        return H(k + ":" + s);
    }

    /**
     * Let HMAC(k, s) be the 16 octet HMAC-MD5 [RFC 2104] of the octet
     * string s using the octet string k as a key.
     */
    private static final byte[] HMAC(String k, String s) {
        // H(K XOR opad, H(K XOR ipad, text))
        // ipad = the byte 0x36 repeated B times
        // opad = the byte 0x5C repeated B times.
        // B = 64 for MD5
        // L = 16 for MD5
        int  B  = 64;
        
        if (s == null) {
            s = "";
        }
        
        byte[] kb = toBytes(k);
        byte[] sb = toBytes(s);
        
        if (kb.length > B) {
            kb = H(kb);
        }
        
        byte[] k_ipad = new byte[B];
        byte[] k_opad = new byte[B];
        for (int i=0; i<B; i++) {
            k_ipad[i] = (byte) (kb[i] ^ 0x36);
            k_opad[i] = (byte) (kb[i] ^ 0x5c);
        }
        
        byte[] inner = H(k_ipad, sb);
        
        return H(k_opad, inner);
    }
    
    private static final byte[] A1(Properties cmap) {
        ByteArrayOutputStream   bout = new ByteArrayOutputStream();
        String                  realm, user, pass, azid, nonce, cnonce;
        byte[]                  temp;
        byte[]                  colon = toBytes(":");

        // If authzid is specified, then A1 is
        //  A1 = { H( { username-value, ":", realm-value, ":", passwd } ),
        //   ":", nonce-value, ":", cnonce-value, ":", authzid-value }
        // If authzid is not specified, then A1 is
        //  A1 = { H( { username-value, ":", realm-value, ":", passwd } ),
        //   ":", nonce-value, ":", cnonce-value }        
    
        //Setup locals
        realm = cmap.getProperty("realm", "");
        user = cmap.getProperty("username", "");
        pass = cmap.getProperty("password", "");
        azid = cmap.getProperty("authzid", "");
        nonce = cmap.getProperty("nonce", "");
        cnonce = cmap.getProperty("cnonce", "");
        
        //Generate cnonce
        if (!Utilities.isValidString(cnonce)) {
            temp = new byte[1024];
            new SecureRandom().nextBytes(temp);
            cnonce = ByteCodec.BASE64.encode(DigestHash.MD5.hashToBytes(ByteCodec.BASE64.encode(temp)));
            cmap.setProperty("cnonce", cnonce);
        }
        
        //Hash username, realm, and password
        temp = H(user + ":" + realm + ":" + pass);
        bout.write(temp, 0, temp.length);
        
        //Append nonce
        temp = toBytes(nonce);
        bout.write(colon, 0, colon.length);
        bout.write(temp, 0, temp.length);
        
        //Append cnonce
        temp = toBytes(cnonce);
        bout.write(colon, 0, colon.length);
        bout.write(temp, 0, temp.length);
        
        if (Utilities.isValidString(azid)) {
            //Append authzid
            temp = toBytes(azid.toString());
            bout.write(colon, 0, colon.length);
            bout.write(temp, 0, temp.length);
        }
        
        return bout.toByteArray();
    }
    private static final String A2(Properties cmap) {
        ByteArrayOutputStream   bout = new ByteArrayOutputStream();
        String                  domain, server, qop;
        StringBuffer            uri = new StringBuffer();
        StringBuffer            retval = new StringBuffer();

        // If the "qop" directive's value is "auth", then A2 is:
        //  A2       = { "AUTHENTICATE:", digest-uri-value }
        // If the "qop" value is "auth-int" or "auth-conf" then A2 is:
        //  A2       = { "AUTHENTICATE:", digest-uri-value,
        //   ":00000000000000000000000000000000" }

        //Setup locals
        domain = cmap.getProperty("host", "");
        server = cmap.getProperty("serv-name", "");
        qop = cmap.getProperty("qop", "auth");
        
        //Setup URI
        uri.append("xmpp").append("/").append(domain);
        if (Utilities.isValidString(server) && !Utilities.equateStrings(domain, server))
            uri.append("/").append(server);
        cmap.setProperty("digest-uri", uri.toString());
        
        retval.append("AUTHENTICATE:").append(uri);
        if (!Utilities.equateStrings(qop, "auth"))
            retval.append(":00000000000000000000000000000000");
        
        return retval.toString();
    }
    
    private static final String RESP(Properties cmap) {
        StringBuffer            s = new StringBuffer();
        String                  nonce, nc, cnonce, qop, a1, a2;
        byte[]                  kd;

        // HEX( KD ( HEX(H(A1)),
        //  { nonce-value, ":" nc-value, ":",
        //    cnonce-value, ":", qop-value, ":", HEX(H(A2)) }))
       
        //Setup nc-count
        nc = "00000001";
        cmap.setProperty("nc", nc);
        
        //Setup locals
        a1 = toHex(H(A1(cmap)));
        a2 = toHex(H(A2(cmap)));
        
        nonce = cmap.getProperty("nonce", "");
        cnonce = cmap.getProperty("cnonce", "");
        qop = cmap.getProperty("qop", "auth");
        
        //Generate kd
        s.append(nonce).append(":").append(nc);
        s.append(":").append(cnonce);
        s.append(":").append(qop);
        s.append(":").append(a2);
        kd = KD(a1, s.toString());
        
        return toHex(kd);
    }
    
    protected boolean isQOPSupported(Properties cmap) {
        String[]    qopOpts;
        String      qop, opt;
        int         idx;
        boolean     supported = false;

        qop = cmap.getProperty("qop", "auth");
        if (!Utilities.isValidString(qop)) {
            supported = true;
        } else {
            qopOpts = qop.split(",");
            for (idx = 0; !supported && (idx < qopOpts.length); idx++) {
                opt = qopOpts[idx].trim();

                supported = Utilities.equateStrings(opt, "auth");
            }
        }

        if (supported) {
            cmap.setProperty("qop", "auth");
        }
        return supported;
    }
    protected boolean configureMaxBuffer(Properties cmap) {
        String  val = cmap.getProperty("maxbuf", Integer.toString(DEFAULT_MAXBUFFER));
        int     maxbuf;
        boolean configed = true;
        

        if (!Utilities.isValidString(val)) {
            try {
                maxbuf = Integer.parseInt(val);
                cmap.setProperty("maxbuf", Integer.toString(maxbuf));
            } catch (NumberFormatException nfe) {
                configed = false;
            }
        }
        
        return configed;
    }
    
    protected SASLPacket generateAuth() {
        SASLAuthPacket sasl = (SASLAuthPacket)getDataFactory().createPacketNode(
                SASLAuthPacket.NAME,
                SASLAuthPacket.class);
        
        sasl.setMechanismName(getMechanismName());
        setCurrentStatus(AUTHENTICATING);
        
        return sasl;
    }
    
    protected SASLPacket evaluateChallenge(SASLPacket challenge) {
        StreamDataFactory   sdf = getDataFactory();
        SASLPacket          resp = null;
        Properties          cmap;
        Status              current = getCurrentStatus();
        
        cmap = MD5MessageCodec.decode(challenge.getData());
        if (current == AUTHENTICATING) {
            resp = evaluateInitialChallenge(cmap);
        } else if   (current == AUTHENTICATING_MORE) {
            resp = evaluateNextChallenge(cmap);
        } else {
            resp = (SASLPacket)sdf.createPacketNode(SASLPacket.NAME_ABORT, SASLPacket.class);
        }
        
        return resp;
    }

    private final SASLPacket evaluateInitialChallenge(Properties cmap) {
        SASLPacket      output = null;
        Object          user, pass, domain, server;
        String          realm, nonce, cnonce;
        String          uri;
        String          qop = "auth";
        List            calls = new java.util.LinkedList();
        JID             azid = getAuthorizationID();
        CallbackHandler cbh = getCallbackHandler();

        user = getProperty("javax.security.sasl.username");
        pass = getProperty("javax.security.sasl.password");
        domain = getProperty("javax.security.sasl.domain");
        server = getProperty("javax.security.sasl.server");

        realm = cmap.getProperty("realm");
        nonce = cmap.getProperty("nonce");
        
        if (domain == null)
            domain = getServer();
        if (server == null)
            server = getServer();
        
        if (cbh != null) {
            if (azid == null)
                calls.add(new AuthorizationIDCallback(CB_AUTHZID));
            if (user == null)
                calls.add(new NameCallback(CB_USERNAME));
            if (pass == null)
                calls.add(new PasswordCallback(CB_PASSWORD, false));
            if (domain == null)
                calls.add(new TextInputCallback(CB_DOMAIN));
            if (server == null)
                calls.add(new TextInputCallback(CB_SERVER));
            if (realm == null)
                calls.add(new RealmInputCallback(CB_REALM));

            if (!calls.isEmpty()) {
                try {
                    Callback    cb;
                    Callback[]  cbs = (Callback[])calls.toArray(new Callback[calls.size()]);
                    Iterator    itr;

                    cbh.handle(cbs);

                    itr = calls.iterator();
                    while (itr.hasNext()) {
                        cb = (Callback)itr.next();

                        if          (cb instanceof NameCallback) {
                            user = ((NameCallback)cb).getName();
                        } else if   (cb instanceof PasswordCallback) {
                            pass = new String(((PasswordCallback)cb).getPassword());
                        } else if   (cb instanceof AuthorizationIDCallback) {
                            azid = ((AuthorizationIDCallback)cb).getAuthorizationID();
                        } else if   (cb instanceof TextInputCallback) {
                            TextInputCallback   ticb = (TextInputCallback)cb;
                            String              prompt = ticb.getPrompt();
                            
                            if          (Utilities.equateStrings(prompt, CB_DOMAIN)) {
                                domain = ticb.getText();
                            } else if   (Utilities.equateStrings(prompt, CB_SERVER)) {
                                server = ticb.getText();
                            }
                        } else if   (cb instanceof RealmInputCallback) {
                            realm = ((RealmInputCallback)cb).getRealm();
                        }
                    }
                } catch (Exception e) {
                    obtainLogger().warn("Could not process all calbacks", e);
                    output = (SASLPacket)getDataFactory().createPacketNode(SASLPacket.NAME_ABORT, SASLPacket.class);
                }
            }
        }
        
        //Verify QOP is "auth" only
        if (!isQOPSupported(cmap)) {
            obtainLogger().warn("Requested qop not supported");
            output = (SASLPacket)getDataFactory().createPacketNode(SASLPacket.NAME_ABORT, SASLPacket.class);
            
            return output;
        }
        
        //Verify "maxbuf" is in range
        if (!configureMaxBuffer(cmap)) {
            obtainLogger().warn("Requested maxbuf not correct");
            output = (SASLPacket)getDataFactory().createPacketNode(SASLPacket.NAME_ABORT, SASLPacket.class);
        }
        
        //Update challenge map
        cmap.setProperty("username", (user != null) ? user.toString() : "");
        cmap.setProperty("password", (pass != null) ? pass.toString() : "");
        cmap.setProperty("realm", (realm != null) ? realm.toString() : "");
        cmap.setProperty("serv-name", (server != null) ? server.toString() : "");
        cmap.setProperty("host", (domain != null) ? domain.toString() : "");
        cmap.setProperty("authzid", (azid != null) ? azid.toString() : "");
        cmap.setProperty("response", RESP(cmap));
        
        if (output == null) {
            output = (SASLPacket)getDataFactory().createPacketNode(SASLPacket.NAME_RESPONSE, SASLPacket.class);
            
            output.setData(MD5MessageCodec.encode(cmap));
            setCurrentStatus(AUTHENTICATING_MORE);
        }
        
        return output;
    }
    private final SASLPacket evaluateNextChallenge(Properties cmap) {
        StreamDataFactory   sdf = getDataFactory();
        
        setCurrentStatus(AUTHENTICATING_MORE);
        
        return (SASLPacket)sdf.createPacketNode(SASLPacket.NAME_RESPONSE, SASLPacket.class);
    }
}