/*
 * ErrorElement.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.List;

/**
 * <p>
 * Base interface for representing an errors (both stream- and
 * stanza-level).</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface ErrorElement extends StreamElement {
    //Methods
    /**
     * <p>
     * Retrieves the textual description for this <tt>ErrorElement</tt>. This
     * value corresponds to the <text/> element. This text is localized
     * specifically to the entity it originates from.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The textual description.
     * @since   JSO-0.12
     */
    public String getText();
    /**
     * <p>
     * Sets the textual description for this <tt>ErrorElement</tt>.</p>
     *
     * <p>
     * The value of <tt>text</tt> may be <tt>null</tt> or "", in which case
     * any previous value is removed.</p>
     *
     * @param text The new textual description.
     * @since   JSO-0.12
     */
    public void setText(String text);
    
    /**
     * <p>
     * Retrieves the list of error conditions for this
     * <tt>ErrorElement</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is an <tt>ErrorCondition</tt> instance.</p>
     *
     * @return The list of contained <tt>ErrorCondition</tt> instances.
     */
    public List listConditions();
    
    /**
     * <p>
     * Retrieves the name of the defined condition for this
     * <tt>ErrorElement</tt>. This is the name of the element matching a
     * defined stream- or stanza-level error condition. This value can be used
     * with {@link #getFirstElement(String, String, Class)} (or any of the
     * appropriate variants) to retrieve the actual <tt>ErrorCondition</tt>
     * instance.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, but may be ""
     * if a defined condition has not been set.</p>
     *
     * @return The name of the defined condition.
     * @since   JSO-0.12
     */
    public String getDefinedCondition();
    /**
     * <p>
     * Sets the name of the defined condition for this
     * <tt>ErrorElement</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case any
     * previous defined condition is removed. If the value of <tt>name</tt>
     * otherwise does not equal a defined condition, an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The name of the defined condition.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void setDefinedCondition(String name) throws IllegalArgumentException;
    
    /**
     * <p>
     * Retrieves the application-specific error condition for this
     * <tt>ErrorElement</tt>. This condition is represented as an element
     * within the <tt>ErrorElement</tt>, that is not in either the
     * <tt>ErrorElement</tt> instance's namespace, in the streams-namespace, or
     * in the stanzas-namespace. This value can be used with {@link
     * #getFirstElement(NSI, Class)} (or any of the appropriate variants) to
     * retrieve the actual <tt>ErrorCondition</tt> instance.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The application-specific error condition.
     * @since   JSO-0.12
     */
    public NSI getApplicationCondition();
    /**
     * <p>
     * Sets the application-specific error condition for this
     * <tt>ErrorElement</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt>, in which case any
     * previous valud is removed. The value returned by {@link
     * NSI#getLocalName()} for <tt>name</tt> cannot be <tt>null</tt> or "", or
     * an <tt>IllegalArgumentException</tt> is thrown. The value returned by
     * {@link NSI#getNamespaceURI()} for <tt>name</tt> cannot be <tt>null</tt>,
     * "", or in this <tt>ErrorElement</tt> instance's namespace, or an
     * <tt>IllegalArgumentException</tt> is thrown. The qualified name
     * represented by <tt>name</tt> cannot be equivalent to a defined condition
     * for this <tt>ErrorCondition</tt> instance, or an
     * <tt>IllegalArgumentException</tt> may be thrown.</p>
     *
     * @param name The new application-specific error condition name.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void setApplicationCondition(NSI name) throws IllegalArgumentException;
}