/*
 * Packet.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.util.List;

import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Interface for all "primary" JABBER elements within the XML stream.  The
 * "core" <tt>Packet</tt> elements are:
 * <ul>
 * <li>InfoQuery (iq)</li>
 * <li>Message</li>
 * <li>Presence</li>
 * </ul>
 * This interface defines the methods that are common to (almost) all primary
 * elements.</p>
 *
 * <p>
 * Of important note is that the following methods are considered "unsupported"
 * for most instances of this interface:
 * <ul>
 * <li><tt>{@link #insert}</tt></li>
 * </ul></p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public interface Packet extends StreamElement {
    //Inner-classes
    /**
     * <p>
     * Base class for enumerating the possible types for a <tt>Packet</tt>.
     * Each specialization of <tt>Packet</tt> provides its own list of
     * types, as "static final" fields.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.2
     */
    public static class Type extends Enumerator {
        //Constructors
        /**
         * <P>
         * Constructs a <tt>Type</tt> for the given type name.</p>
         *
         * <p>
         * The value of <tt>name</tt> cannot be <tt>null</tt> or "",
         * otherwise an <tt>IllegalArgumentException</tt> is thrown.</p>
         *
         * @param name Name of the type.
         * @throws IllegalArgumentException If the parameter is invalid.
         * @since   JSO-0.2
         */
        protected Type(String name) throws IllegalArgumentException {
            super(name);
        }
    }

    //"Constants"
    /** Represents the "error" type for most <tt>Packet</tt> objects.
     * @since   JSO-0.2
     */
    public static final Type    ERROR = new Type("error");

    //Methods
    /**
     * <p>
     * Retrieves the type for this <tt>Packet</tt>.  Each
     * <tt>Packet</tt> has its own set of allowed types, defined as
     * subclasses to <tt>Type</tt>.  Depending on the type, different
     * data may be present, or a different interpretation of the data is
     * necessary.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>, if it has
     * not been set, is not required, or is the default for the specific
     * instance of <tt>Packet</tt> or its derivatives.</p>
     *
     * @return The type for this <tt>Packet</tt>.
     * @since   JSO-0.2
     */    
    public Type getType();
    /** <p>
     * Sets the type for this <tt>Packet</tt>.Each <tt>Packet</tt> has
     * its own set of allowed types, defined as subclasses to
     * <tt>PacketType</tt>.</p>
     *
     * <p>
     * This method cannot be used directly to set the type to <tt>{@link
     * #ERROR}</tt>, and an <tt>IllegalArgumentException</tt>
     * is thrown if this is the value of <tt>type</tt>.  Instead, use the
     * method <tt>{@link #setError}</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> must be one of the allowed types for
     * the specific implementation of <tt>Packet</tt> in use, or an
     * <tt>IllegalArgumentException</tt> is thrown.  The value of
     * <tt>type</tt> cannot be <tt>null</tt> unless allowed by the
     * specific implementation.</p>
     *
     * @param type The new type for this <tt>Packet</tt>.
     * @throws IllegalArgumentException If the value of <tt>type</tt> is
     * not acceptable.
     * @since   JSO-0.2
     */
    public void setType(Type type) throws IllegalArgumentException;

    /**
     * <p>
     * Retrieves the <tt>PacketError</tt> for this <tt>Packet</tt>.
     * This method should only return a non-null value if the value from
     * <tt>{@link #getType}</tt> is "error".</p>
     *
     * <p>
     * For this <tt>Packet</tt> to be valid for output, this method
     * must return a valid <tt>PacketError</tt> if
     * <tt>{@link #getType}</tt> returns <tt>{@link #ERROR}</tt>.</p>
     *
     * @return The <tt>PacketError</tt> for this <tt>Packet</tt>.
     * @since   JSO-0.2
     */    
    public PacketError getError();
    /**
     * <p>
     * Sets the <tt>PacketError</tt> for this <tt>Packet</tt>.
     * This method ensures that the current value for <tt>{@link
     * #getType}</tt> matches.</p>
     *
     * <p>
     * If the value of <tt>err</tt> is not <tt>null</tt>, this method
     * also sets the type to <tt>{@link #ERROR}</tt>.  Otherwise, if the
     * type was "error" and <tt>err</tt> is <tt>null</tt>, this method
     * "clears" the type (if allowed; if the type is not allowed to be
     * <tt>null</tt> and <tt>err</tt> is <tt>null</tt>, then this
     * method may throw an <tt>IllegalArgumentException</tt>).</p>
     *
     * @param err The new <tt>PacketError</tt>.
     * @throws IllegalArgumentException If the parameter is not valid.
     * @since   JSO-0.2
     */    
    public void setError(PacketError err) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and sets a <tt>PacketError</tt> for this <tt>Packet</tt> with
     * the given error code.</p>
     *
     * <p>
     * The value of <tt>code</tt> cannot be less than 0, or an
     * <tt>IllegalArugmentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param code The error code.
     * @return The newly added <tt>PacketError</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     * @deprecated As of JSO-0.12, this method is replaced with {@link
     * #setError(PacketError.Type, String)}.
     */
    public PacketError setError(int code) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and sets a <tt>PacketError</tt> for this <tt>Packet</tt> with
     * the given error code and message.</p>
     *
     * <p>
     * The value of <tt>code</tt> cannot be less than 0, or an
     * <tt>IllegalArugmentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>msg</ttM may be <tt>null</tt> or "", in which case it
     * is ignored.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param code The error code.
     * @param msg The error message.
     * @return The newly added <tt>PacketError</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     * @deprecated As of JSO-0.12, this method is replaced with {@link
     * #setError(PacketError.Type, String)}.
     */
    public PacketError setError(int code, String msg) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and sets a <tt>PacketError</tt> with the given type and defined
     * condition for this <tt>Packet</tt>. This method also results in {@link
     * #getType} returning {@link #ERROR}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>cond</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "undefined-condition". If the value of <tt>cond</tt>
     * does not match a defined stanza condition, this method throws an
     * <tt>IllegalArgumentException</tt>.</p>
     *
     * @param type The type of error.
     * @param cond The defined error condition.
     * @return The newly added <tt>PacketError</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public PacketError setError(PacketError.Type type, String cond) throws IllegalArgumentException;
    
    /**
     * <p>
     * Retrieves the "from" <tt>JID</tt> for this <tt>Packet</tt>.
     * The definition of "from" is dependent upon which stream direction this
     * <tt>Packet</tt> is for.  If this is from the input
     * stream, "from" is the <tt>JID</tt> for the remote side.
     * Otherwise, the <tt>JID</tt> is for the local side.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if
     * the "from" <tt>JID</tt> is not defined.  This will not be
     * the case for <tt>Packet</tt> objects generated from the
     * input stream.</p>
     *
     * @return The "from" <tt>JID</tt>.
     * @since   JSO-0.2
     */    
    public JID getFrom();
    /**
     * <p>
     * Sets the "from" <tt>JID</tt> for this <tt>Packet</tt>.
     * If the value of <tt>from</tt> is <tt>null</tt>, the current
     * <tt>JID</tt> is removed.</p>
     *
     * @param from The new "from" <tt>JID</tt>.
     * @since   JSO-0.2
     */
    public void setFrom(JID from);
    
    /**
     * <p>
     * Retrieves the "to" <tt>JID</tt> for this <tt>Packet</tt>.
     * The definition for "to" is dependent upon which stream direction this
     * <tt>Packet</tt> is for.  If this is from the input
     * stream, "to" is the <tt>JID</tt> for the remote side.
     * Otherwise, the <tt>JID</tt> is for the local side.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if
     * the "to" <tt>JID</tt> is not defined.  This will not be
     * the case for <tt>Packet</tt> objects generated from the
     * input stream.</p>
     *
     * @return The "to" <tt>JID</tt>.
     * @since   JSO-0.2
     */    
    public JID getTo();
    /**
     * <p>
     * Sets the "to" <tt>JID</tt> for this <tt>Packet</tt>.
     * If the value of <tt>to</tt> is <tt>null</tt>, the current
     * <tt>JID</tt> is removed.</p>
     *
     * @param to The new "to" <tt>JID</tt>.
     * @since   JSO-0.2
     */
    public void setTo(JID to);

    /**
     * <p>
     * Lists the <tt>Extension</tt> objects that are contained by this
     * <tt>Packet</tt>.  This list consists of all the
     * <tt>Extension</tt> children, regardless of namespace.  The ordering
     * of this list follows the order they are located.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.  The
     * returned list may or may not be modifiable.</p>
     *
     * @return The list of contained <tt>Extension</tt> objects.
     * @since   JSO-0.2
     */
    public List listExtensions();
    /**
     * <p>
     * Lists the <tt>Extension</tt> objects that are contained by this
     * <tt>Packet</tt> that impelement the given interface. This list consists
     * of all the <tt>Extension</tt> children (regardless of namespace) that
     * are assignable to <tt>iface</tt>. The ordering of this list follows the
     * order they are located.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt> in which case it is
     * ignored. The class <tt>iface</tt> must implement or extend
     * <tt>Extension</tt>, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.  The
     * returned list may or may not be modifiable.</p>
     *
     * @param iface The class each <tt>Extension</tt> is assignable from.
     * @return The list of matching <tt>Extension</tt> objects.
     * @since   JSO-0.12
     */
    public List listExtensions(Class iface) throws IllegalArgumentException;
    /**
     * <p>
     * Lists the <tt>Extension</tt> objects athat are contained by this
     * <tt>Packet</tt> for the given namespace. This list consists of all the
     * <tt>Extension</tt> children of the given namespace URI.  The
     * ordering of this list follows the order that they are located.</p>
     *
     * <p>
     * The value of <tt>uri</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of <tt>{@link #listExtensions()}</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.  The
     * returned list is not generally modifiable.</p>
     *
     * @param uri  The namespace URI to filter by.
     * @return the list of matching <tt>Extension</tt> objects.
     * @since   JSO-0.2
     */
    public List listExtensions(String uri);
    /**
     * <p>
     * Lists the <tt>Extension</tt> objects that are contained by this
     * <tt>Packet</tt> for the given namespace and/or interface. This list
     * consists of all the <tt>Extension</tt> children of the given namespace
     * URI, and implementing the given interface. The ordering of this list
     * follows the order that they are located.</p>
     *
     * <p>
     * The value of <tt>uri</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of <tt>{@link #listExtensions()}</tt>.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt> in which case it is
     * ignored. The class <tt>iface</tt> must implement or extend
     * <tt>Extension</tt>, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.  The
     * returned list is not generally modifiable.</p>
     *
     * @param uri  The namespace URI to filter by.
     * @param iface The class each <tt>Extension</tt> is assignable from.
     * @return The list of matching <tt>Extension</tt> objects.
     * @since   JSO-0.12
     */
    public List listExtensions(String uri, Class iface) throws IllegalArgumentException;
    
    /**
     * <p>
     * Retrieves the first <tt>Extension</tt> contained by this
     * <tt>Packet</tt>, regardless of namespace.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no
     * <tt>Extension</tt> could be found.</p>
     *
     * @return The first contained <tt>Extension</tt>.
     * @since   JSO-0.12
     */
    public Extension getExtension();
    /**
     * <p>
     * Retrieves the first <tt>Extension</tt> contained by this
     * <tt>Packet</tt> that implements the given interface.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt> in which case it is
     * ignored. The calss <tt>iface</tt> must implement or extend
     * <tt>Extension</tt>, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no
     * <tt>Extension</tt> could be found.</p>
     *
     * @param iface The class the <tt>Extension</tt> is assignable to.
     * @return The first matching <tt>Extension</tt>.
     * @since   JSO-0.12
     */
    public Extension getExtension(Class iface) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the first <tt>Extension</tt> contained by this
     * <tt>Packet</tt> for the given namespace.</p>
     *
     * <p>
     * The value of <tt>uri</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of {@link #getExtension()}.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no
     * <tt>Extension</tt> could be found.</p>
     *
     * @param uri  The namespace URI to filter by.
     * @return The first matching <tt>Extension</tt>.
     * @since   JSO-0.12
     */
    public Extension getExtension(String uri);
    /**
     * <p>
     * Retrieves the first <tt>Extension</tt> contained by this
     * <tt>Packet</tt> for the given namespace and/or interface.</p>
     *
     * <p>
     * The value of <tt>uri</tt> can be <tt>null</tt> or "", in which
     * case the behavior is that of {@link #getExtension()}.</p>
     *
     * <p>
     * The value of <tt>iface</tt> can be <tt>null</tt> in which case it is
     * ignored. The calss <tt>iface</tt> must implement or extend
     * <tt>Extension</tt>, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no
     * <tt>Extension</tt> could be found.</p>
     *
     * @param uri  The namespace URI to filter by.
     * @param iface The class the <tt>Extension</tt> is assignable to.
     * @return The first matching <tt>Extension</tt>.
     * @since   JSO-0.12
     */
    public Extension getExtension(String uri, Class iface) throws IllegalArgumentException;

    /**
     * <p>
     * Adds the given <tt>Extension</tt> to this <tt>Packet</tt>.
     * This method appends <tt>x</tt> to the end of the current array
     * of <tt>Extension</tt> objects, and attaches the registered
     * <tt>VetoableChangeListener</tt> (if any).</p>
     *
     * <p>
     * If <tt>x</tt> is <tt>null</tt>, this method does nothing.</p>
     *
     * @param x The new <tt>Extension</tt> to add.
     * @since   JSO-0.2
     */
    public void addExtension(Extension x);
    /**
     * <p>
     * Removes the given <tt>Extension</tt> from this <tt>Packet</tt>.
     * This method finds <tt>x</tt> and clears it from the current array
     * of <tt>Extension</tt> objects, and detaches the registered
     * <tt>VetoableChangeListener</tt> (if any).</p>
     *
     * <p>
     * If <tt>x</tt> is <tt>null</tt>, this method does nothing.</p>
     *
     * @param x The <tt>Extension</tt> to remove.
     * @since   JSO-0.2
     */
    public void removeExtension(Extension x);
}