/*
 * PacketError.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Represents an error within a <tt>Packet</tt>. Each <tt>PacketError</tt>
 * instance includes a general type categorization (such as {@link #CANCEL}),
 * a defined condition (such as {@link #BAD_REQUEST_CONDITION}, and an optional
 * application-specific condition.</p>
 * 
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public interface PacketError extends ErrorElement, Comparable {
    //Inner-classes
    /**
     * <p>
     * Class for enumerating the possible types for a <tt>PacketError</tt>.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static final class Type extends Enumerator {
        //Constructors
        private Type(String name) {
            super(name);
        }
    }

    //Constants
    /**
     * Namespace URI for stanza-level error conditions.
     *
     * @since   JSO-0.12
     */
    public static final String  CONDITION_NAMESPACE = "urn:ietf:params:xml:ns:xmpp-stanzas";
    
    /** Type to indicate a warning.
     *
     * @since   JSO-0.12
     */
    public static final Type    CONTINUE    = new Type("continue");
    /** Type to indicate a temporary error state.
     *
     * @since   JSO-0.12
     */
    public static final Type    WAIT        = new Type("wait");
    /** Type to indicate invalid data.
     *
     * @since   JSO-0.12
     */
    public static final Type    MODIFY      = new Type("modify");
    /** Type to indicate unauthorized access.
     *
     * @since   JSO-0.12
     */
    public static final Type    AUTH        = new Type("auth");
    /** Type to indicate an unrecoverable error.
     *
     * @since   JSO-0.12
     */
    public static final Type    CANCEL      = new Type("cancel");
    
    /** Condition for indicating the XML is invalid (but still well-formed).
     * The associated type is usually {@link #MODIFY}.
     *
     * @since   JSO-0.12
     */
    public static final String  BAD_REQUEST_CONDITION               = "bad-request";
    /** Condition for indicating a conflict. The associated type is usually
     * {@link #CANCEL}.
     *
     * @since   JSO-0.12
     */
    public static final String  CONFLICT_CONDITION                  = "conflict";
    /** Condition for indicating a feature is not implemented. The associated
     * type is usually {@link #CANCEL}.
     *
     * @since   JSO-0.12
     */
    public static final String  FEATURE_NOT_IMPLEMENTED_CONDITION   = "feature-not-implemented";
    /** Condition for indicating a forbidden action. The associated type is
     * usually {@link #AUTH}.
     *
     * @since   JSO-0.12
     */
    public static final String  FORBIDDEN_CONDITION                 = "forbidden";
    /** Condition for indicating an entity that is permanently relocated or
     * removed. The associated type is usually {@link #MODIFY}.
     *
     * @since   JSO-0.12
     */
    public static final String  GONE_CONDITION                      = "gone";
    /** Condition for indicating the server encountered an internal error. The
     * associated type is usually {@link #WAIT}.
     *
     * @since   JSO-0.12
     */
    public static final String  INTERNAL_SERVER_ERROR_CONDITION     = "internal-server-error";
    /** Condition for indicating an item cannot be located. The associated type
     * is usually {@link #CANCEL}.
     *
     * @since   JSO-0.12
     */
    public static final String  ITEM_NOT_FOUND_CONDITION            = "item-not-found";
    /** Condition for indicating a provided JID is not well-formed. The
     * associated type is usually {@link #MODIFY}.
     *
     * @since   JSO-0.12
     */
    public static final String  JID_MALFORMED_CONDITION             = "jid-malformed";
    /** Condition for indicating a request that, while understood, is not
     * permitted. The associated type is usually {@link #CANCEL}.
     *
     * @since   JSO-0.12
     */
    public static final String  NOT_ACCEPTABLE_CONDITION            = "not-acceptable";
    /** Condition for indicating an entity is not allowed to perform an action.
     * The associated type is usually {@link #CANCEL}.
     *
     * @since   JSO-0.12
     */
    public static final String  NOT_ALLOWED_CONDITION               = "not-allowed";
    /** Condition for indicating an entity is not authorized to perform an
     * action. The associated type is usually {@link #AUTH}.
     *
     * @since   JSO-0.12
     */
    public static final String  NOT_AUTHORIZED_CONDITION            = "not-authorized";
    /** Condition for indicating the action requires payment before it can be
     * performed. The associated type is usually {@link #AUTH}.
     *
     * @since   JSO-0.12
     */
    public static final String  PAYMENT_REQUIRED_CONDITION          = "payment-required";
    /** Condition for indicating an entity that is temporarily not available.
     * The associated type is usually {@link #WAIT}.
     *
     * @since   JSO-0.12
     */
    public static final String  RECIPIENT_UNAVAILABLE_CONDITION     = "recipient-unavailable";
    /** Condition for indicating an entity that is temporarily relocated or
     * removed. The associated type is usually {@link #MODIFY}.
     *
     * @since   JSO-0.12
     */
    public static final String  REDIRECT_CONDITION                  = "redirect";
    /** Condition for indicating an entity must be registered before the action
     * can be performed. The associated type is usually {@link #AUTH}.
     *
     * @since   JSO-0.12
     */
    public static final String  REGISTRATION_REQUIRED_CONDITION     = "registration-required";
    /** Condition for indicating the server on the receiving end does not
     * exist. The associated type is usually {@link #CANCEL}.
     *
     * @since   JSO-0.12
     */
    public static final String  REMOTE_SERVER_NOT_FOUND_CONDITION   = "remote-server-not-found";
    /** Condition for indicating the server on the receiving end could not be
     * contacted within some time limit. The associated type is usually {@link
     * #WAIT}.
     *
     * @since   JSO-0.12
     */
    public static final String  REMOTE_SERVER_TIMEOUT_CONDITION     = "remote-server-timeout";
    /** Condition for indicating the processing entity does not currently have
     * the resources to perform the action. The associated type is usually
     * {@link #WAIT}.
     *
     * @since   JSO-0.12
     */
    public static final String  RESOURCE_CONSTRAINT_CONDITION       = "resource-constraint";
    /** Condition for indicating an entity does not provide the requested
     * service. The associated type is usually {@link #CANCEL}.
     *
     * @since   JSO-0.12
     */
    public static final String  SERVICE_UNAVAILABLE_CONDITION       = "service-unavailable";
    /** Condition for indicating an entity must be subscribed to the requested
     * item before the action can be performed. The associated type is usually
     * {@link #AUTH}.
     *
     * @since   JSO-0.12
     */
    public static final String  SUBSCRIPTION_REQUIRED_CONDITION     = "subscription-required";
    /** Condition for indicating an unknown or application-specific error has
     * occured. An application-specific <tt>ErrorCondition</tt> should be
     * present.
     *
     * @since   JSO-0.12
     */
    public static final String  UNDEFINED_CONDITION                 = "undefined-condition";
    /** Condition for indicating the request was not expected at this time. The
     * associated type is usually {@link #WAIT}.
     *
     * @since   JSO-0.12
     */
    public static final String  UNEXPECTED_REQUEST_CONDITION        = "unexpected-request";

    //"Constants" -- 3xx Errors
    /** Any redirection error within Jabber.  This error is not currently
     * reported.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     REDIRECT = 302;
    //"Constants" -- 4xx Errors
    /** A semantic error in the "request".  This results from such conditions
     * as sending a message with no "to".
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     BAD_REQUEST = 400;
    /** An invalid username and/or password was used.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     UNATHORIZED = 401;
    /** Payment is required.  This error is not currently reported.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     PAYMENT_REQUIRED = 402;
    /** The server understood the request, but cannot fulfill it.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     FORBIDDEN = 403;
    /** The node of the "to" JID cannot be found.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     NOT_FOUND = 404;
    /** The action is not allowed.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     NOT_ALLOWED = 405;
    /** The request was not acceptable.  This results from such conditions
     * as missing registration information, or invalid namespaces.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     NOT_ACCEPTABLE = 406;
    /** The server did not receive an expected request in time.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     REQUEST_TIMEOUT = 408;
    /** A conflict with the provided information was found.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     CONFLICT = 409;
    //"Constants" -- 5xx Errors
    /** An unexpected condition occured trying to handle the last request.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     INTERNAL_SERVER_ERROR = 500;
    /** The request cannot be handled because it is not understood.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     NOT_IMPLEMENTED = 501;
    /** The remote server for a request cannot be reached.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     REMOTE_SERVER_ERROR = 502;
    /** The service is not (currently) available.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     SERVICE_UNAVAILABLE = 503;
    /** The remote server did not respond.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * classe/condition instead.
     */    
    public static final int     REMOTE_SERVER_TIMEOUT = 504;
    
    //Methods
    /**
     * <p>
     * Retrieves the type for this <tt>PacketError</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The type of error.
     * @since   JSO-0.12
     */
    public Type getType();
    /**
     * <p>
     * Sets the type for this <tt>PacketError</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param type The new type.
     * @since   JSO-0.12
     */
    public void setType(Type type);
    
    /**
     * <p>
     * Retrieves the numeric code for this <tt>PacketError</tt>.  This
     * code is roughly equivalent to the HTTP error of the same meaning.</p>
     *
     * The value returned by this method is never less than 0.</p>
     *
     * @return The numeric code for this <tt>PacketError</tt>.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * type and condition instead.
     */    
    public int getCode();
    /**
     * <p>
     * Sets the numeric code for this <tt>PacketError</tt>.</p>
     *
     * <p>
     * The value of <tt>code</tt> cannot be less than 0, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param code The new code for this <tt>PacketError</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     * @deprecated  As of JSO-0.12, "code" is no longer supported. Use error
     * type and condition instead.
     */
    public void setCode(int code) throws IllegalArgumentException;
}