/*
 * StreamContext.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


import java.nio.charset.Charset;

/**
 * <p>
 * Encapsultes the detailed information for a specific "direction" of a
 * <tt>Stream</tt>.  The direction is either the inbound ("remote")
 * or the outbound ("local") &lt;stream:stream/&gt;.  This interface
 * contains all the information from the &lt;stream:stream/&gt; element
 * (including declared namespaces), and the associated
 * <tt>StreamDataFactory</tt> and <tt>PacketRouter</tt> instances.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public interface StreamContext extends StreamElement {
    //Methods
    /**
     * <p>
     * Retrieves the <tt>StreamDataFactory</tt> for this <tt>StreamContext</tt>.
     * This is used to generate the various <tt>StreamNode</tt> objects, related
     * (in some manner) to this <tt>StreamContext</tt>.</p>
     *
     * <p>
     * Using this <tt>StreamDataFactory</tt> is preferrable over any other
     * obtained reference, as this instance may automatically initialize created
     * objects with information from this <tt>StreamContext</tt>.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The data factory for this <tt>StreamContext</tt>.
     * @since   JSO-0.10
     */
    public StreamDataFactory getDataFactory();

    /**
     * <p>
     * Retrieves the <tt>PacketRouter</tt> for this <tt>StreamContext</tt>.
     * This value is used to send <tt>Packet</tt>s associated to this
     * <tt>StreamContext</tt>.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * @return The <tt>PacketRouter</tt> for this <tt>StreamContext</tt>.
     * @since   JSO-0.10
     */
    public PacketRouter getRouter();
    
    /**
     * <p>
     * Determines if the direction for this <code>StreamContext</code> is
     * "inbound".  "Inbound" means the data travels from the remote end of the
     * connection to this local end of the connection.</p>
     *
     * <p>
     * This method returns <code>true</code> if this <code>StreamContext</code>
     * is for the inbound connection, regardless of the current state of the
     * connection.</p>
     *
     * @return <code>true</code> if this is for the inbound connection.
     * @since   JSO-0.2
     */
    public boolean isInbound();
    /**
     * <p>
     * Determines if the direction for this <code>StreamContext</code> is
     * "outbound".  "Outbound" means the data travels from this local end of
     * the connection to the remote end of the connection.</p>
     *
     * <p>
     * This method returns <code>true</code> if this <code>StreamContext</code>
     * is for the outbound connection, regardless of the current state of the
     * connection.</p>
     *
     * @return <code>true</code> if this is for the outbound connection.
     * @since   JSO-0.2
     */
    public boolean isOutbound();
    
    /**
     * <p>
     * Retrieves the current status for this <tt>StreamContext</tt>. The
     * status determines the connnectivity for this "direction" on the owning
     * <tt>Stream</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The current status for this <tt>StreamContext</tt>.
     * @since   JSO-0.10
     */
    public Stream.Status getCurrentStatus();
    
    /**
     * <p>
     * Retrieves the name of the character-set for this <tt>StreamContext</tt>.
     * This is the charset used for either encoding (when {@link #isOutbound}
     * is <tt>true</tt>) or decoding (when {@link #isInbound} is
     * <tt>true</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, but may be
     * "" if it has not yet been set.</p>
     *
     * @return The character-set name.
     * @since JSO-0.10
     */
    public String getCharsetName();
    /**
     * <p>
     * Sets the name of the character-set for this <tt>StreamContext</tt>.
     * If this method completes successfully, this method will clear any cached
     * <tt>Charset</tt> reference.</p>
     *
     * <p>
     * The value of <tt>name</tt> must be the name of a supported character-set,
     * or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The new character-set name.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since JSO-0.10
     */
    public void setCharsetName(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Obtains a valid <tt>Charset</tt> object for this <tt>StreamContext</tt>.
     * This method uses the following algorithm to retrieve the
     * <tt>Charset</tt>:</p>
     *
     * <ol>
     * <li>Use the cached <tt>Charset</tt> reference.</li>
     * <li>Obtain a <tt>Charset</tt> for {@link #getCharsetName}, if
     * defined.</li>
     * <li>Obtain a <tt>Charset</tt> for "UTF-8".</li>
     * </ol>
     *
     * <p>
     * If the obtained <tt>Charset</tt> is cached if not already. If the value
     * of {@link #getCharsetName} returned <tt>null</tt>, then {@link
     * #setCharsetName} is called with "UTF-8".</p>
     *
     * @return The <tt>Charset</tt> for this <tt>StreamContext</tt>.
     * @since JSO-0.10
     */
    public Charset obtainCharset();
    
    /**
     * <p>
     * Retrieves the "from" <code>JID</code> for this
     * <code>StreamContext</code>.  The usage of this method depends on
     * the direction of this <Code>StreamContext</code>, as this may represent
     * this end of the <code>Stream</code>, or the remote end.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>.</p>
     *
     * @return The "from" <code>JID</code> for this <code>StreamContext</code>.
     * @since   JSO-0.2
     */
    public JID getFrom();
    /**
     * <p>
     * Sets the "from" <code>JID</code> for this <code>StreamContext</code>.
     * Calling this method after its attached <code>Stream</code> is connected
     * or accepted may have deterimental effects on the connection.  Depending
     * on how this instance of <Code>StreamContext</code> is obtained, this
     * method may effectively do nothing.</p>
     *
     * <p>
     * The value of <code>from</code> can be <code>null</code>.</p>
     *
     * @param from The new "from" <code>JID</code>.
     * @since   JSO-0.2
     */
    public void setFrom(JID from);
    
    /**
     * <p>
     * Retrieves the "to" <code>JID</code> for this <Code>StreamContext</code>.
     * The usage of this method depends on the direction of this
     * <code>StreamContext</code>, as this may represent the remote end of the
     * <code>Stream</code>, or this end.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>.</p>
     *
     * @return The "to" <code>JID</code> for this <Code>StreamContext</code>.
     * @since   JSO-0.2
     */
    public JID getTo();
    /**
     * <p>
     * Sets the "to" <code>JID</code> for this <Code>StreamContext</code>.
     * Calling this method after its attached <code>Stream</code> is connected
     * or accepted may have detrimental effects on the connection.  Depending
     * on how this instance of <code>StreamContext</code> is obtained, this
     * method may effectively do nothing.</p>
     *
     * <p>
     * The value of <code>to</code> can be <code>null</code>.</p>
     *
     * @param to The new "to" <code>JID</code>.
     * @since   JSO-0.2
     */
    public void setTo(JID to);
    
    /**
     * <p>
     * Retrieves the version of this <tt>StreamContext</tt>. This value
     * corresponds to the "version" attribute of the &lt;stream:stream/&gt;.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. The returned
     * value may be "" if there is no version.</p>
     *
     * @return The version of this <tt>StreamContext</tt>.
     * @since   JSO-0.12
     */
    public String getVersion();
    /**
     * <p>
     * Sets the version of this <tt>StreamContext</tt>.</p>
     *
     * <p>
     * The value of <tt>ver</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.</p>
     *
     * @param ver The new version.
     * @since   JSO-0.12
     */
    public void setVersion(String ver);
}