/*
 * PacketDispatcher.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;

/**
 * <p>
 * "Registration" interface for objects that dispatch <code>PacketEvent</code>.
 * This interface provides the methods to register and unregister
 * <code>PacketListener</code> objects.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.6
 */
public interface PacketDispatcher {
    //Methods
    /**
     * <p>
     * Registers the given <code>PacketListener</code> for any packet
     * transfer events.  The given listener is registered to events for both
     * received and sent <code>Packet</code> objects.</p>
     *
     * <p>
     * The value of <Code>l</code> should not be <code>null</code>.  No actions
     * are performed if this is true.  If <code>l</code> is already registered,
     * it may again be registered.</p>
     *
     * @param l The <code>PacketListener</code> to register.
     * @since   JSO-0.6
     */
    public void addPacketListener(PacketListener l);
    /**
     * <p>
     * Registers the given <code>PacketListener</code> for packet transfer
     * events of the given type.  The given listener is registered only to
     * events matching <Code>t</code>.</p>
     *
     * <p>
     * The value of <Code>t</code> can be <code>null</code>, in which case this
     * method performs identically to {@link
     * #addPacketListener(PacketListener)}.</p>
     *
     * <p>
     * The value of <Code>l</code> should not be <code>null</code>.  No actions
     * are performed if this is true.  If <code>l</code> is already registered,
     * it is registered again.</p>
     *
     * @param type The <code>PacketEvent.Type</code> to register for.
     * @param l The <code>PacketListener</code> to register.
     * @since   JSO-0.6
     */
    public void addPacketListener(PacketEvent.Type type, PacketListener l);
    
    /**
     * <p>
     * Unregisters the given <code>PacketListener</code> for any packet
     * transfer events.  The given listener is unregistered from events for both
     * received and sent <code>Packet</code> objects.</p>
     *
     * <p>
     * The value of <Code>l</code> should not be <code>null</code>.  No actions
     * are performed if this is true.  If <code>l</code> is registered more
     * than once, only one is unregistered.</p>
     *
     * @param l The <code>PacketListener</code> to unregister.
     * @since   JSO-0.6
     */
    public void removePacketListener(PacketListener l);
    /**
     * <p>
     * Unegisters the given <code>PacketListener</code> for packet transfer
     * events of the given type.  The given listener is unregistered only from
     * events matching <Code>t</code>.</p>
     *
     * <p>
     * The value of <Code>t</code> can be <code>null</code>, in which case this
     * method performs identically to {@link
     * #removePacketListener(PacketListener)}.</p>
     *
     * <p>
     * The value of <Code>l</code> should not be <code>null</code>.  No actions
     * are performed if this is true.  If <code>l</code> is registered more
     * than once, only one is unregistered.</p>
     *
     * @param type The <code>PacketEvent.Type</code> to register for.
     * @param l The <code>PacketListener</code> to register.
     * @since   JSO-0.6
     */
    public void removePacketListener(PacketEvent.Type type, PacketListener l);
}