/*
 * FeaturesetConsumerManager.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.features;


import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.StreamError;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.StreamFeatureset;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.event.StreamFeaturesEvent;
import org.jabberstudio.jso.event.StreamFeaturesListener;
import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Class used to manage the consumption of a <tt>StreamFeatureset</tt>.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class FeaturesetConsumerManager implements Runnable {
    //Inner-classes
    /**
     * <p>
     * Class to enumerate the status types for a <tt>FeatureConsumer</tt>.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static class ConsumerStatus extends Enumerator {
        //Fields
        private boolean     _Started;
        private boolean     _Complete;
        
        //Constructors
        private ConsumerStatus(String name, boolean started, boolean complete) {
            super(name);
            
            _Started = started;
            _Complete = complete;
        }
        
        //Methods
        /**
         * <p>
         * Determines if this <tt>Status</tt> corresponds to a started
         * state.</p>
         *
         * @return <tt>true</tt> if the flagged <tt>FeatureConsumer</tt> is
         * started.
         * @since   JSO-012
         */
        public final boolean isStarted() {
            return _Started;
        }
        /**
         * <p>
         * Determines if this <tt>Status</tt> corresponds to a completed
         * state.</p>
         *
         * @return <tt>true</tt> if the flagged <tt>FeatureConsumer</tt> is
         * completed.
         * @since   JSO-012
         */
        public final boolean isComplete() {
            return _Complete;
        }
    }
    private class Context implements FeatureContext, StreamFeaturesListener {
        //Fields
        private FeatureConsumer _Consumer;
        private StreamFeature   _Feature;
        private boolean         _Pending;
        
        //Constructors
        public Context() {
        }
        
        //Methods
        public FeatureConsumer getConsumer() {
            return _Consumer;
        }
        public StreamFeature getFeature() {
            return _Feature;
        }
        public void start(FeatureConsumer fc, StreamFeature feat) {
            _Consumer = fc;
            _Feature = feat;
            
            updateConsumerStatus(getConsumer(), IN_PROGRESS);
        }
        
        public PacketRouter getRouter() {
            return _Conn;
        }
        public PacketDispatcher getDispatcher() {
            return _Conn;
        }
        
        public void completeFailed(Throwable t) {
            setFailure(t);
            updateConsumerStatus(getConsumer(), FAILED);
        }
        public void completeSucceeded(boolean restart) {
            updateConsumerStatus(getConsumer(), SUCCEEDED);

            try {
                if (restart) {
                    setFeaturesPending(true);
                    
                    getConnection().drop();
                    getConnection().open();
                } else {
                    getFeatureset().remove(getFeature());
                    process(getFeatureset());
                }
            } catch (Throwable t) {
                setFailure(t);
            }
        }
        
        public void featuresReported(StreamFeaturesEvent evt) {
            setFeaturesPending(false);
            
            process(evt.getFeatureset());
        }
    }
    
    
    //"Constants"
    /**
     * Status for a <tt>FeatureConsumer</tt> that is ready to start consuming.
     *
     * @since   JSO-0.12
     */
    public static final ConsumerStatus READY        = new ConsumerStatus("ready", false, false);
    /**
     * Status for a <tt>FeatureConsumer</tt> that is in the process of
     * consuming.
     *
     * @since   JSO-0.12
     */
    public static final ConsumerStatus IN_PROGRESS  = new ConsumerStatus("in progress", true, false);
    /**
     * Status for a <tt>FeatureConsumer</tt> that failed during consumption.
     *
     * @since   JSO-0.12
     */
    public static final ConsumerStatus FAILED       = new ConsumerStatus("failed", true, true);
    /**
     * Status for a <tt>FeatureConsumer</tt> that successfully completed
     * consumption.
     *
     * @since   JSO-0.12
     */
    public static final ConsumerStatus SUCCEEDED    = new ConsumerStatus("succeeded", true, true);
    
    //Fields
    private Stream                  _Conn;

    private boolean                 _Pending;
    private StreamFeatureset        _Featureset;
    private Throwable               _Failure;
    private Context                 _Ctx;
    
    private List                    _Consumers = new java.util.LinkedList();
    private Map                     _Armed = new java.util.TreeMap();
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>FeaturesetConsumerManager</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public FeaturesetConsumerManager() {
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>Stream</tt> connection for this
     * <tt>FeaturesetConsumerManager</tt>.</p>
     *
     * <p>
     * This method may return <tt>null</tt> if it is not currently attached to
     * a <tt>Stream</tt>.</p>
     *
     * @return The attached <tt>Stream</tt>.
     */
    public synchronized Stream getConnection() {
        return _Conn;
    }
    /**
     * <p>
     * Attaches this <tt>FeaturesetConsumerManager</tt> to the given
     * <tt>Stream</tt>. Attaching also resets this
     * <tt>FeaturesetConsumerManager</tt>.</p>
     *
     * <p>
     * Note that a <tt>FeaturesetConsumerManager</tt> can only be attached to a
     * single <tt>Stream</tt> at a time. If a second attempt is made, an
     * <tt>IllegalStateException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>conn</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param conn The <tt>Stream</tt> to attach to.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @throws IllegalStateException If this <tt>FeatureConsumerManager</tt> is
     * already attached.
     * @since   JSO-0.12
     */
    public synchronized void attach(Stream conn) throws IllegalArgumentException, IllegalStateException {
        if (conn == null)
            throw new IllegalArgumentException("Stream cannot be null");
        if (_Conn != null)
            throw new IllegalStateException("FeaturesetConsumerManager is already attached");
        _Conn = conn;

        if (_Ctx == null) {
            _Ctx = new Context();
        }
        _Conn.addStreamFeaturesListener(PacketEvent.RECEIVED, _Ctx);
    }
    /**
     * <p>
     * Detaches this <tt>FeaturesetConsumerManager</tt> from its current
     * <tt>Stream</tt>.
     *
     * @since   JSO-0.12
     */
    public synchronized void detach() {
        reset();

        _Conn.removeStreamFeaturesListener(PacketEvent.RECEIVED, _Ctx);
        _Conn = null;
    }
    /**
     * <p>
     * Resets this <tt>FeaturesetConsumerManager</tt>. This method clears
     * the current failure (if any), clears the status for all registered
     * <tt>FeatureConsumer</tt>s, and calls {@link FeatureConsumer#reset} on
     * each registered <tt>FeatureConsumer</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void reset() {
        Iterator        itr;
        
        synchronized (this) {
            itr = listFeatureConsumers().iterator();
            _Failure = null;
            _Ctx = null;
            _Armed.clear();
        }
        
        while (itr.hasNext())
            ((FeatureConsumer)itr.next()).reset();
    }
    
    /**
     * <p>
     * Lists the <tt>FeatureConsumer</tt> objects currently registered with
     * this <tt>FeaturesetConsumerManager</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is an instance of <tt>FeatureConsumer</tt>. The
     * returned list cannot be modified.</p>
     *
     * @return The list of registered <tt>FeatureConsumer</tt>s.
     */
    public synchronized List listFeatureConsumers() {
        return new java.util.ArrayList(_Consumers);
    }
    /**
     * <p>
     * Registers the given <tt>FeatureConsumer</tt>.</p>
     *
     * <p>
     * The value of <tt>fc</tt> may be <tt>null</tt>, in which case no action
     * is performed. Also, if <tt>fc</tt> is already registered, it is not
     * re-registered.</p>
     *
     * @param fc The <tt>FeatureConsumer</tt> to register.
     */
    public synchronized void registerFeatureConsumer(FeatureConsumer fc) {
        if ((fc != null) && !_Consumers.contains(fc))
            _Consumers.add(fc);
    }
    /**
     * <p>
     * Unregisters the given <tt>FeatureConsumer</tt>. Unregistering a 
     * <tt>FeatureConsumer</tt> also removes any remembered status.</p>
     *
     * <p>
     * The value of <tt>fc</tt> may be <tt>null</tt>, in which case no action
     * is performed.</p>
     *
     * @param fc The <tt>FeatureConsumer</tt> to unregister.
     */
    public synchronized void unregisterFeatureConsumer(FeatureConsumer fc) {
        if (fc != null) {
            _Consumers.remove(fc);
            _Armed.remove(fc.toString());
        }
    }

    /**
     * <p>
     * Determines the current status for the given
     * <tt>FeatureConsumer</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>fc</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, <tt>fc</tt> must be
     * currently registered, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @return The current status for the <tt>FeatureConsumer</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public synchronized ConsumerStatus getConsumerStatus(FeatureConsumer fc) throws IllegalArgumentException {
        ConsumerStatus      stat = null;
        
        if (!_Consumers.contains(fc))
            throw new IllegalArgumentException("FeatureConsumer is not registered");
        
        if (fc != null) {
            stat = (ConsumerStatus)_Armed.get(fc.toString());
        }
        
        if (stat == null)
            stat = READY;
        
        return stat;
    }
    private final synchronized void updateConsumerStatus(FeatureConsumer fc, ConsumerStatus stat) throws IllegalArgumentException {
        String      id;
        
        if (!_Consumers.contains(fc))
            throw new IllegalArgumentException("FeatureConsumer is not registered");

        id = fc.toString();
        if ((stat != null) && (stat != READY))
            _Armed.put(id, stat);
        else
            _Armed.remove(id);
    }
    /**
     * <p>
     * Determines if the given <tt>FeatureConsumer</tt> has started.</p>
     *
     * <p>
     * The value of <tt>fc</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, <tt>fc</tt> must be
     * currently registered, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param fc The <tt>FeatureConsumer</tt> to check.
     * @return <tt>true</tt> if the given <tt>FeatureConsumer</tt> has started.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public boolean isFeatureConsumerStarted(FeatureConsumer fc) throws IllegalArgumentException {
        return getConsumerStatus(fc).isStarted();
    }
    /**
     * <p>
     * Determines if the given <tt>FeatureConsumer</tt> has completed.</p>
     *
     * <p>
     * The value of <tt>fc</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, <tt>fc</tt> must be
     * currently registered, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param fc The <tt>FeatureConsumer</tt> to check.
     * @return <tt>true</tt> if the given <tt>FeatureConsumer</tt> has
     * completed.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public boolean isFeatureConsumerCompleted(FeatureConsumer fc) throws IllegalArgumentException {
        return getConsumerStatus(fc).isComplete();
    }
    
    private synchronized final Context getFeatureContext() {
        return _Ctx;
    }
    private synchronized final Context startFeatureContext(FeatureConsumer fc, StreamFeature feat) {
        if (_Ctx == null)
            _Ctx = new Context();
        _Ctx.start(fc, feat);
        
        return _Ctx;
    }
    
    private synchronized final StreamFeatureset getFeatureset() {
        return _Featureset;
    }
    private synchronized final void setFeatureset(StreamFeatureset fs) {
        _Featureset = fs;
    }

    /**
     * <p>
     * Determines if this <tt>FeaturesetConsumerManager</tt> has been started.
     * This method returns <tt>true</tt> if at least one
     * <tt>FeatureConsumer</tt> has been started, or if {@link #getFailure}
     * does not return <tt>null</tt>.</p>
     *
     * @return <tt>true</tt> if this <tt>FeaturesetConsumerManager</tt> has
     * been started.
     * @since   JSO-0.12
     */
    public boolean isStarted() {
        return (!_Armed.isEmpty() || (getFailure() != null));
    }
    /**
     * <p>
     * Determines if this <tt>FeaturesetConsumerManager</tt> has completed.
     * This method returns <tt>true</tt> if all started
     * <tt>FeatureConsumer</tt>s have completed, or if {@link #getFailure}
     * does not return <tt>null</tt>.</p>
     *
     * @return <tt>true</tt> if this <tt>FeaturesetConsumerManager</tt> has
     * been completed.
     * @since   JSO-0.12
     */
    public boolean isCompleted() {
        return isCompleted(false);
    }
    /**
     * <p>
     * Determines if this <tt>FeaturesetConsumerManager</tt> has completed.
     * This method returns <tt>true</tt> if (possibly) all registered
     * <tt>FeatureConsumer</tt>s have completed, or if {@link #getFailure} does
     * not return <tt>null</tt>.</p>
     *
     * <p>
     * If the value of <tt>all</tt> is <tt>true</tt>, then this method returns
     * <tt>true</tt> if all registered <FeatureConsumer</tt>s have completed.
     * Otherwise, the results are the same as for {@link #isCompleted}.</p>
     *
     * @param all <tt>true</tt> if all registered <tt>FeatureConsumer</tt>s
     * should be completed.
     * @return <tt>true</tt> if this <tt>FeaturesetConsumerManager</tt> has
     * been completed.
     * @since   JSO-0.12
     */
    public boolean isCompleted(boolean all) {
        Iterator        itr = _Armed.values().iterator();
        boolean         completed = !_Armed.isEmpty();
        
        if (getFailure() != null)
            return true;
        
        if (all)
            completed = (completed && (_Consumers.size() == _Armed.size()));
        while (completed && itr.hasNext()) {
            completed = (completed && ((ConsumerStatus)itr.next()).isComplete());
        }
        
        return completed;
    }
    
    /**
     * <p>
     * Retrieves the failure cause for this
     * <tt>FeaturesetConsumerManager</tt>.</p>
     *
     * <p>
     * This method may return <tt>null</tt>.</p>
     *
     * @return The cause of failure.
     * @since   JSO-0.12
     */
    public synchronized Throwable getFailure() {
        return _Failure;
    }
    private final synchronized void setFailure(Throwable t) {
        _Failure = t;
    }
    
    /**
     * <p>
     * Process the given <tt>StreamFeatureset</tt>.</p>
     *
     * @param fs The <tt>StreamFeatureset</tt> to process.
     * @since   JSO-0.12
     */
    protected void process(StreamFeatureset fs) {
        Iterator        fcItr = listFeatureConsumers().iterator();
        FeatureConsumer fc = null;
        StreamFeature   feat = null;
        Context         ctx;
        StreamError     se;
        
        setFeatureset(fs);
        
        //Find a feature to consume
        while ((feat == null) && fcItr.hasNext()) {
            fc = (FeatureConsumer)fcItr.next();
            feat = fc.findFeature(fs);
        }

        try {
            if          (feat != null) {
                //Consume the feature
                ctx = startFeatureContext(fc, feat);
                
                fc.consume(ctx);
            } else if   (!isCompleted()) {
                //Could not start or finish
                se = getConnection().getDataFactory().createStreamError(StreamError.POLICY_VIOLATION_CONDITION);
                
                throw new StreamException(se);
            }
        } catch (Throwable t) {
            setFailure(t);
        }
    }

    /**
     * <p>
     * Runs this <tt>FeaturesetConsumerManager</tt> until all startable
     * consumers complete. Any errors that occur are reported via
     * {@link #getFailure}.</p>
     *
     * @since   JSO-0.12
     */
    public synchronized void run() {
        try {
            setFeaturesPending(true);
            do {
                getConnection().process();
                
                try {
                    wait(10);
                } catch (InterruptedException ie) {
                    
                }
            } while (isFeaturesPending() || !isCompleted());
            setFeaturesPending(false);
        } catch (Throwable t) {
            setFailure(t);
        }
    }

    private boolean isFeaturesPending() {
        return _Pending;
    }
    private void setFeaturesPending(boolean pending) {
        _Pending = pending;
    }
}