/*
 * InputStreamBuffer.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.io.src;


import java.io.IOException;
import java.io.InputStream;
import java.net.SocketTimeoutException;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;

import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Provides "non-blocking" usage of a <code>InputStream</code>. This class uses
 * a separate thread to read from a supplied <tt>StreamInput</tt> into a
 * temporary buffer.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.10
 */
public class InputStreamQueue {
    //Fields
    private String              _Name;
    private Thread              _Runner;
    private IOException         _Caught;
    private boolean             _Continue;
    private boolean             _Stopped;
    
    private InputStream         _Input;
    private byte[]              _Trans;

    private ByteBuffer          _Output;
    private int                 _Avail;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>InputStreamQueue</tt> using the given thread name,
     * <tt>InputStream</tt>, and buffer size.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * it is set to "jso-queuedinputbuffer-&lt;hash of toString()&gt;".</p>
     *
     * <p>
     * The value of <tt>in</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException is thrown.</p>
     *
     * <p>
     * The value of <tt>size</tt> must be a positive integer, or an
     * <tt>IllegalArgumentException</tt> is thown.</p>
     *
     * @param name The thread name.
     * @param in The <tt>InputStream</tt> to read from.
     * @param size The size of the internal buffer.
     * @throws IllegalArgumentException If any paremeters are invalid.
     * @since   JSO-0.10
     */
    public InputStreamQueue(String name, InputStream in, int size) throws IllegalArgumentException {
        if (in == null)
            throw new IllegalArgumentException("InputStream cannot be null");
        _Name = (Utilities.isValidString(name)) ? name : "jso-queuedinputstream-" + String.valueOf(toString().hashCode());
        _Input = in;
        _Caught = null;
        _Continue = false;
        
        if (size < 1)
            throw new IllegalArgumentException("size cannot be less than 1");
        _Trans = new byte[size];
        _Output = ByteBuffer.allocate(size);
        _Output.flip();
    }
    
    private final Thread getRunner() {
        return _Runner;
    }

    //Methods
    private synchronized boolean isStopped() {
        return _Stopped;
    }
    private synchronized void setStopped(boolean val) {
        _Stopped = val;
        notifyAll();
    }
    
    /**
     * <p>
     * Determines if this <tt>InputStreamQueue</tt> is still running.</p>
     *
     * @return <tt>true</tt> if still reading data.
     * @since   JSO-0.10
     */
    public final synchronized boolean isRunning() {
        return _Continue;
    }
    private final synchronized void beginRunning() {
        _Continue = true;
    }
    private final synchronized void endRunning(boolean force, Throwable e) {
        IOException     caught = null;

        if (_Continue) {
            if          (e instanceof IOException) {
                caught = (IOException)e;
            } else if   (e != null) {
                caught = new IOException("unknown I/O error");
                caught.initCause(e);
            }

            _Continue = false;
            
            _Caught = caught;
            if ((_Runner != null) && force) {
                _Runner.interrupt();
                _Stopped = true;
            }
            _Runner = null;
        }
        notifyAll();
    }

    /**
     * <p>
     * Retrieves the number of bytes available to be read from this
     * <tt>InputStreamQueue</tt>.</p>
     *
     * @return The number of bytes available.
     * @since   JSO-0.10
     */
    public final int getAvailable() {
        return _Avail;
    }
    private final void setAvailable(int avail) {
        _Avail = avail;
    }
    
    private final synchronized void checkCaught() throws IOException {
        if (_Caught != null)
            throw _Caught;
    }

    /**
     * <p>
     * Determines if this <tt>InputStreamQueue</tt> is valid. This method
     * returns <tt>true</tt> if either the input thread is running or there
     * are bytes available from retrieval.</p>
     *
     * @return <tt>true</tt> If <tt>InputStreamQueue</tt> is valid.
     * @since   JSO-0.10
     */
    public final synchronized boolean isValid() {
        return isRunning() || (getAvailable() > 0);
    }

    /**
     * <p>
     * Retrieves any available data. This method copies data from the internal
     * buffer into the give byte array, using the provided offset and
     * length.</p>
     *
     * <p>
     * The value of <tt>buffer</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>offset</tt> cannot be less than 0, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>length</tt> cannot be less than 0, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * An <tt>IllegalArgumentException</tt> may be thrown if the offset and
     * length exceed the the length of <tt>buffer</tt>.</p>
     *
     * @param buffer The array to read to.
     * @param offset The first index of <tt>buffer</tt> to read to.
     * @param length The maximum amount of data to read.
     * @return The number of bytes read, or -1 if EOF has occurred.
     * @throws IOException If any I/O errors occur.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.10
     */
    public final synchronized int read(byte[] buffer, int offset, int length) throws IllegalArgumentException, IOException {
        ByteBuffer  output = _Output;
        int         amt = 0;
        
        //Validate parameters
        if (buffer == null)
            throw new IllegalArgumentException("buffer cannot be null");
        if ((offset < 0) || ((buffer.length - length) < offset))
            throw new IllegalArgumentException("offset is not within valid range");
        if ((length < 0) || ((buffer.length + offset) > length))
            throw new IllegalArgumentException("length is not within valid range");

        if          (output.remaining() > 0) {
            //Get the input
            amt = Math.min(output.remaining(), length);
            output.get(buffer, offset, amt);
        } else if (!isRunning()) {
            //Check for caught exceptions
            checkCaught();
            //Putting in the fork
            amt = -1;
        }

        return amt;
    }
    private final int transfer() throws IOException {
        InputStream     is = _Input;
        ByteBuffer      buffer = _Output;
        byte[]          temp;
        int             data;
        int             amt, rem, avail;
        
        if (is == null)
            throw new IOException("not connected");

        //Check for remaining
        synchronized (this) {
            amt = buffer.remaining();
        }
        
        if (amt == 0) {
            //Read any available data
            temp = _Trans;
            try {
                amt = is.read(temp, 0, temp.length);
            } catch (SocketTimeoutException ste) {
                amt = 0;
            }
            
            if (amt > 0) {
                //Prep buffer for consumption
                synchronized (this) {
                    buffer.compact();
                    buffer.put(temp, 0, amt);
                    buffer.flip();
                }
            }
        }
        
        return amt;
    }

    /**
     * <p>
     * Starts this <tt>InputStreamQueue</tt>. This method creates and starts
     * a new Thread to read from the provided <tt>InputStream</tt>.</p>
     *
     * @since   JSO-0.10
     */
    public synchronized void start() {
        if (isRunning())
            return;
        
        //Initialize and start thread
        _Runner = new Thread(_Name) {
            public void run() {
                try {
                    while (isRunning()) {
                        if (transfer() == -1)
                            endRunning(false, null);
                        //This keeps us out of a tight loop
                        try { Thread.sleep(10); } catch (InterruptedException ie) {}
                    }
                } catch (IOException ioe) {
                    endRunning(false, ioe);
                } finally {
                    synchronized (InputStreamQueue.this) { setStopped(true); }
                }
            }
        };
        
        _Runner.setDaemon(true);
        _Runner.start();
        beginRunning();
    }
    /**
     * <p>
     * Stops this <tt>InputStreamQueue</tt>. This method flags completion on
     * the running internal Thread, then forces an interrupt.</p>
     *
     * @since   JSO-0.10
     */
    public synchronized void stop() {
        endRunning(true, null);
    }
    /**
     * <p>
     * Stops this <tt>InputStreamQueue</tt>. This method flags completion on
     * the running internal Thread.</p>
     *
     * <p>
     * If the value of <tt>force</tt> is <tt>true</tt>, the internal thread is
     * interrupted.</p>
     *
     * @param force <tt>true</tt> if the internal thread should be interruped.
     * @since   JSO-0.10
     */
    public synchronized void stop(boolean force) {
        endRunning(force, null);
    }
    
    public synchronized void stopAndWait(boolean force) {
        endRunning(force, null);
        
        while (!isStopped()) {
            try {
                wait();
            } catch (InterruptedException ie) {
                
            }
        }
    }
}