/*
 * Enumerator.java
 *
 * Created on October 9, 2003, 8:24 AM
 */

package org.jabberstudio.jso.util;

/**
 * <p>
 * Utility class to represent enumerations.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class Enumerator implements Comparable, java.io.Serializable {
    //Fields
    private final   String  _Name;
    private final   int     _Code;
    
    private static int      LAST_CODE = 0;
    
    //Constructors
    /**
     * <P>
     * Constructs a <tt>Enumerator</tt> for the given name.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "",
     * otherwise an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name Name of the <tt>Enumerator</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    protected Enumerator(String name) throws IllegalArgumentException {
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("Type name cannot be null or \"\"");
        _Name = name;
        _Code = nextCode();
    }

    //Methods
    private static final synchronized int nextCode() {
        return ++LAST_CODE;
    }
    
    /**
     * <p>
     * Retrieves the code for this <tt>Enumerator</tt>. The code is used
     * for natural ordering, and can be used with switch statements.</p>
     *
     * @return The code for this <tt>Enumerator</tt>.
     * @since   JSO-0.12
     */
    public final int getCode() {
        return _Code;
    }
    
    /**
     * <p>
     * Generates or retrieves the string representing this
     * <tt>Enumerator</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * @return The string representing this <tt>Enumerator</tt>.
     * @since   JSO-0.12
     */
    public String toString() {
        return _Name;
    }
    /**
     * <p>
     * Generates or retrieves the hash code for this <tt>Enumerator</tt>.</p>
     *
     * @return The hash code for this <tt>Enumerator</tt>.
     * @since   JSO-0.12
     */
    public int hashCode() {
        return (31 * 23) + toString().hashCode();
    }
    
    /**
     * <p>
     * Compares this <tt>Enumerator</tt> to the given object for equality.
     * This method uses object reference comparisons if the types match, and
     * otherwise attempts to use the string representation of each.</p>
     *
     * <p>
     * The value of <tt>cmp</tt> may be <tt>null</tt>, in which case this
     * method always returns <tt>false</tt>.</p>
     *
     * @param cmp The object to compare to.
     * @return <tt>true</tt> if the two objects are equivalent.
     * @since   JSO-0.12
     */
    public boolean equals(Object cmp) {
        boolean     results = (cmp != null);
        
        if (results) {
            results = toString().equals(cmp.toString());
        }
        
        return results;
    }
    
    /**
     * <p>
     * Compares this <tt>Enumerator</tt> to ghet given object for natural
     * ordering. This method uses the value of {@link #getCode} to perform
     * the comparison.</p>
     *
     * <p>
     * The value returned by this method is less than 0 if <tt>cmp</tt> is
     * after, 0 if their oerdering is the same, and greater than 0 if
     * <tt>cmp</tt> is before this <tt>Enumerator</tt>.</p>
     *
     * <p>
     * The value of <tt>cmp</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown. If <tt>cmp</tt> is not
     * an instance of <tt>Enumerator</tt>, a <tt>ClassCastException</tt>
     * is thrown.</p>
     *
     * @param cmp The object to compare against.
     * @return The relative ordering.
     * @throws NullPointerException If <tt>cmp</tt> is <tt>null</tt>.
     * @throws ClassCastException If <tt>cmp</tt> is not an instance of
     * <tt>Enumerator</tt>.
     * @since   JSO-0.12
     */
    public int compareTo(Object cmp) {
        Enumerator      test = null;
        int             code1, code2;
        
        if (cmp == null)
            throw new NullPointerException("Object cannot be null");
        if (cmp instanceof Enumerator)
            test = (Enumerator)cmp;
        else
            throw new ClassCastException("Object is not an instance of Enumerator");

        code1 = getCode();
        code2 = test.getCode();
        
        return ((code1 < code2) ? -1 : ((code1 > code2) ? 1 : 0));
    }
}