/*
 * ProhibitedTable.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package org.jabberstudio.jso.util.stringprep;

/**
 * <p>
 * Class for encapsulating a prohibited character table.</p>
 *
 * <p>
 * Subclasses must implement {@link #verify} to check the input string is not
 * prohibited.</p>
 *
 * <p>
 * A default implementation that verifies against a two-dimensional array of
 * characters is provided via {@link #create}.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12.3
 */
public abstract class ProhibitedTable {
    //Constructors
    /**
     * <p>
     * Creates a new <tt>ProhibitedTable</tt>.</p>
     *
     * @since   JSO-0.12.3
     */
    protected ProhibitedTable() {
    }

    //Methods
    /**
     * <p>
     * Verifies the character sequence does is not prohibited.</p>
     *
     * @param   cs The character sequence to verify.
     * @throws  ProhibitedException If <tt>cs</tt> is not valid.
     * @throws  NullPointerException If <tt>cs</tt> is <tt>null</tt>.
     * @since   JSO-0.12.3
     */
    public abstract void verify(CharSequence cs) throws ProhibitedException, NullPointerException;


    /**
     * <p>
     * Creates a new <tt>ProhibitedTable</tt> based on the given table.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. The returned
     * <tt>ProhibitedTable</tt>'s <tt>verify(CharSequence)</tt> iterates
     * through the characters in the provided <tt>CharSequence</tt>, verifying
     * they are not contained by the ranges in <tt>values</tt>.</p>
     *
     * <p>
     * The format of <tt>values</tt> is an array of ranges; each array within
     * <tt>values</tt> is expected to be either single characters (array length
     * is 1) or a min/max range (array length is 2).</p>
     *
     * <p>
     * If <tt>values</tt> is <tt>null</tt>, an <tt>IllegalArgumentException</tt>
     * is thrown. If any of the sub-arrays in <tt>values</tt> are <tt>null</tt>,
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param   values The table of prohibited characters.
     * @return  The <tt>ProhibitedTable</tt> over <tt>values</tt>.
     * @throws  IllegalArgumentException If <tt>values</tt> is not valid.
     * @since   JSO-0.12.3
     */
    public static ProhibitedTable create(final char[][] values) {
        int     idx;

        if (values == null)
            throw new IllegalArgumentException("values arrays cannot be null");

        for (idx = 0; idx < values.length; idx++) {
            if (values[idx] == null)
                throw new IllegalArgumentException("values sub-arrays cannot be null");
        }

        return new ProhibitedTable() {
            public void verify(CharSequence cs) throws ProhibitedException, NullPointerException {
                boolean     result = false;
                char[]      range;
                char        ch;
                int         idx, valIdx = 0;

                for (idx = 0; idx < cs.length(); idx++) {
                    ch = cs.charAt(idx);

                    for (valIdx = 0; valIdx < values.length; valIdx++) {
                        range = values[valIdx];

                        if      (range.length == 1)
                            result = (ch == range[0]);
                        else if (range.length == 2)
                            result = ((ch >= range[0]) && (ch <= range[1]));

                        if (result)
                            throw new ProhibitedException("character sequence contains prohibited characters");
                    }
                }
            }
        };
    }
}