/*
 * AuthFeatureConsumer.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import java.util.Map;

import org.saxpath.SAXPathException;

import org.jaxen.SimpleVariableContext;

import org.jabberstudio.jso.InfoQuery;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.StreamFeatureset;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.features.AbstractFeatureConsumer;
import org.jabberstudio.jso.features.FeatureContext;
import org.jabberstudio.jso.util.IdentityGenerator;
import org.jabberstudio.jso.util.PacketException;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.util.XPathListener;

/**
 * <p>
 * <tt>FeatureConsumer</tt> implementation for consuming non-SASL
 * authentication.</p>
 *
 * <p>
 * <b>NOTE:</b> This implementation does <b>NOT SUPPORT</b> the method {@link
 * AuthQuery#ZERO_KNOWLEDGE}.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class AuthFeatureConsumer extends AbstractFeatureConsumer {
    //Inner-classes
    private class Context implements FeatureContext {
        //Fields
        private StreamFeature       _Feat;
        private PacketRouter        _Router;
        private PacketDispatcher    _Dispatch;
        private Throwable           _Throw;
        
        private boolean             _Running;
        
        //Constructors
        public Context(PacketRouter r, PacketDispatcher d) {
            if (r == null)
                throw new IllegalArgumentException("PacketRouter cannot be null");
            if (d == null)
                throw new IllegalArgumentException("PacketDispatcher cannot be null");

            _Router = r;
            _Dispatch = d;
            _Feat = (StreamFeature)r.getDataFactory().createElementNode(AuthStreamFeature.NAME, AuthStreamFeature.class);
            _Running = true;
        }
        
        //Methods
        public boolean isRunning() {
            return _Running;
        }
        public Throwable getFailure() {
            return _Throw;
        }
        
        public StreamFeature getFeature() {
            return _Feat;
        }
        public PacketRouter getRouter() {
            return _Router;
        }
        public PacketDispatcher getDispatcher() {
            return _Dispatch;
        }
        
        public void completeFailed(Throwable t) {
            _Throw = t;
            _Running = false;
        }
        public void completeSucceeded(boolean restart) {
            _Running = false;
        }
    }
    private class AuthWatcher extends XPathListener {
        //Fields
        private PacketRouter    _Router;
        private int             _CurrState;
        
        //Constructors
        public AuthWatcher(PacketRouter router) throws SAXPathException {
            super(router.getDataFactory(), "app:iq[(@id=$id) and ((@type='result') or (@type='error'))]");
            
            Map     nss = new java.util.HashMap();
            nss.put("app", router.getDefaultNamespace());
            nss.put("auth", AuthQuery.NAMESPACE);
            
            _Router = router;
            setupNamespaces(nss);
            setCurrentState(STATE_INITIAL);
            getXPath().setVariableContext(new SimpleVariableContext());
        }
        
        //Methods
        public PacketRouter getRouter() {
            return _Router;
        }
        
        public int getCurrentState() {
            return _CurrState;
        }
        public void setCurrentState(int state) {
            _CurrState = state;
        }
        public void packetMatched(PacketEvent evt) {
            PacketDispatcher    dispatch = getFeatureContext().getDispatcher();
            InfoQuery           iq = (InfoQuery)evt.getData();
            AuthQuery           auth;

            if (iq.getType() == InfoQuery.ERROR) {
                setCurrentState(STATE_FAILED);
                completeFailedConsumption(new PacketException(iq.getError()));
            } else {
                switch (getCurrentState()) {
                    case STATE_INITIAL:     authStarted(iq);    break;
                    case STATE_AUTH:        authSucceeded(iq);  break;
                    default:
                        //This should never happen
                        completeFailedConsumption(new IllegalStateException("unexpected auth state"));
                }
            }
            
            if ((getCurrentState() == STATE_FAILED) || (getCurrentState() == STATE_SUCCEEDED))
                dispatch.removePacketListener(this);
        }
        
        protected void authStarted(InfoQuery iq) {
            PacketRouter        router = getRouter();
            StreamDataFactory   sdf = router.getDataFactory();
            AuthQuery           auth;
            
            try {
                setCurrentState(STATE_AUTH);
                
                auth = configureAuth(router, (AuthQuery)iq.getExtension(AuthQuery.class));
                auth.setUsername(getUsername());
                auth.setResource(getResource());
                
                iq = (InfoQuery)iq.copy();
                iq.reset();
                iq.setID(IdentityGenerator.generateGlobal("auth"));
                iq.setTo(getServer());
                iq.setType(InfoQuery.SET);
                iq.add(auth);
                
                ((SimpleVariableContext)getXPath().getVariableContext()).setVariableValue("id", iq.getID());
                router.send(iq);
            } catch (Throwable t) {
                setCurrentState(STATE_FAILED);
                completeFailedConsumption(t);
            }
        }
        protected void authSucceeded(InfoQuery iq) {
            setCurrentState(STATE_SUCCEEDED);
            completeSucceededConsumption(false);
        }
    }
    
    //"Constants"
    private static final int    STATE_INITIAL   = 0;
    private static final int    STATE_AUTH      = 1;
    private static final int    STATE_SUCCEEDED = 2;
    private static final int    STATE_FAILED    = -1;
    
    //Fields
    private JID         _Srv;
    
    private String      _Usr;
    private String      _Pwd = "";
    private String      _Res;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>AuthFeatureConsumer</tt> using the default values.</p>
     *
     * <p><b>NOTE:</b> the username, password, and resource <b>MUST</b> be
     * set before attempting to consume a <tt>StreamFeatureset</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public AuthFeatureConsumer() {
    }
    /**
     * <p>
     * Creates a new <tt>AuthFeatureConsumer</tt> with the given username
     * and resource.</p>
     *
     * <p>
     * The value of <tt>srv</tt> should not be <tt>null</tt>, but is not
     * enforced.</p>
     *
     * <p>
     * The value of <tt>usr</tt> must meet the criteria for {@link
     * #setUsername}, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>pwd</tt> may be <tt>null</tt>, in which case it is
     * considered to be "".</p>
     *
     * <p>
     * The value of <tt>res</tt> must meet the criteria for {@link
     * #setResource}, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param srv The server to authenticate with.
     * @param usr The username to authenticate with.
     * @param pwd The password to authenticate with.
     * @param res The resource to authenticate with.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public AuthFeatureConsumer(JID srv, String usr, String pwd, String res) throws IllegalArgumentException {
        setServer(srv);
        setUsername(usr);
        setPassword(pwd);
        setResource(res);
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the server this <tt>AuthFeatureConsumer</tt> will attempt
     * to authenticate with.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has
     * not been set.</p>
     *
     * @return The server to authenticate with.
     * @since   JSO-0.12
     */
    public JID getServer() {
        return _Srv;
    }
    /**
     * <p>
     * Sets the server this <tt>AuthFeatureConsumer</tt> will attempt to
     * authenticate with.</p>
     *
     * <p>
     * The value of <tt>srv</tt> should not be <tt>null</tt>, but this is not
     * enforced.</p>
     *
     * @param srv The new server to authenticate with.
     * @since   JSO-0.12
     */
    public void setServer(JID srv) {
        _Srv = srv;
    }
    
    /**
     * <p>
     * Retrieves the username this <tt>AuthFeatureConsumer</tt> will attempt
     * to authenticate with.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> or "" if it has
     * not been set.</p>
     *
     * @return The username to authenticate with.
     * @since   JSO-0.12
     */
    public String getUsername() {
        return _Usr;
    }
    /**
     * <p>
     * Sets the username this <tt>AuthFeatureConsumer</tt> will attempt to
     * authenticate with.</p>
     *
     * <p>
     * The value of <tt>usr</tt> cannot be "" or <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, <tt>usr</tt> must be
     * valid JID node string, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param usr The new username to authenticate with.
     * @throws IllegalArgumentException If the parameter is not valid.
     * @since   JSO-0.12
     * @see     JID#isValidNode
     */
    public void setUsername(String usr) throws IllegalArgumentException {
        if (!Utilities.isValidString(usr) || !JID.isValidNode(usr))
            throw new IllegalArgumentException("String \"" + usr + "\" is not a valid username");
        _Usr = usr;
    }
    
    /**
     * <p>
     * Retrieves the password this <tt>AuthFeatureConsumer</tt> will attempt
     * to authenticate with.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The password to authenticate with.
     * @since   JSO-0.12
     */
    public String getPassword() {
        return _Pwd;
    }
    /**
     * <p>
     * Sets the password this <tt>AuthFeatureConsumer</tt> will attempt to
     * authenticate with.</p>
     *
     * <p>
     * The value of <tt>pwd</tt> may be <tt>null</tt>, in which case it is
     * considered to be "".</p>
     *
     * @param pwd The new password to authenticate with.
     * @since   JSO-0.12
     */
    public void setPassword(String pwd) {
        _Pwd = ((pwd != null) ? pwd : "");
    }
    
    /**
     * <p>
     * Retrieves the resource this <tt>AuthFeatureConsumer</tt> will attempt
     * to authenticate with.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> or "" if it has
     * not been set.</p>
     *
     * @return The resource to authenticate with.
     * @since   JSO-0.12
     */
    public String getResource() {
        return _Res;
    }
    /**
     * <p>
     * Sets the resource this <tt>AuthFeatureConsumer</tt> will attempt to
     * authenticate with.</p>
     *
     * <p>
     * The value of <tt>res</tt> cannot be "" or <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, <tt>res</tt> must be
     * valid JID resource string, or an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @param res The new resource to authenticate with.
     * @throws IllegalArgumentException If the parameter is not valid.
     * @since   JSO-0.12
     * @see     JID#isValidResource
     */
    public void setResource(String res) throws IllegalArgumentException {
        if (!Utilities.isValidString(res) || !JID.isValidResource(res))
            throw new IllegalArgumentException("String \"" + res + "\" is not a valid resource");
        _Res = res;
    }
    
    /**
     * <p>
     * Authenticates using the given <tt>PacketRouter</tt> and
     * <tt>PacketDispatcher</tt>. This method uses the authentication
     * properties of this <tt>AuthFeatureConsumer</tt>.</p>
     *
     * <p>
     * The value of <tt>router</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>dispatch</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param router The <tt>PacketRouter</tt> to authenticate with.
     * @param dispatch The <tt>PacketDispatcher</tt> to authenticate with.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @throws IllegalStateException If the properties are not valid.
     * @throws StreamException If any stream-level failure occurs.
     * @throws PacketException If any application-level failure occurs.
     * @since   JSO-0.12
     */
    public void authenticate(PacketRouter router, PacketDispatcher dispatch) throws IllegalArgumentException, IllegalStateException, StreamException, PacketException {
        PacketException     pe;
        Context             ctx = new Context(router, dispatch);
        Throwable           cause;

        setFeatureContext(ctx);
        try {
            startAuth(router, dispatch);
        } catch (Throwable t) {
            ctx.completeFailed(t);
        }

        try {
            while (ctx.isRunning()) {
                router.process();
                try {
                    Thread.sleep(10);
                } catch (InterruptedException ie) {
                }
            }
        } catch (StreamException se) {
            ctx.completeFailed(se);
        }
        
        cause = ctx.getFailure();
        if (cause != null) {
            if          (cause instanceof StreamException) {
                throw (StreamException)cause;
            } else if   (cause instanceof PacketException) {
                throw (PacketException)cause;
            } else if   (cause instanceof IllegalStateException) {
                throw (IllegalArgumentException)cause;
            } else {
                pe = new PacketException(PacketError.CANCEL, PacketError.UNDEFINED_CONDITION);
                
                pe.initCause(cause);
                throw pe;
            }
        }
    }
    
    /**
     * <p>
     * Finds the <tt>StreamFeature</tt> in the given <tt>StreamFeatureset</tt>
     * that can be consumed. This implementation searches for the first
     * <tt>StreamElement</tt> with the NSI of {@link AuthStreamFeature#NAME}
     * in <tt>fs</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a consumable
     * <tt>StreamFeature</tt> could not be found.</p>
     *
     * <p>
     * The value of <tt>fs</tt> will never be <tt>null</tt>.</p>
     *
     * @param fs The <tt>StreamFeatureset</tt> to search in
     * @return The consumable <tt>StreamFeature</tt>.
     * @since   JSO-0.12
     */
    public StreamFeature findFeature(StreamFeatureset fs) {
        return (StreamFeature)fs.getFirstElement(AuthStreamFeature.NAME);
    }
    /**
     * <p>
     * Begins non-SASL authentication. This method registers a
     * <tt>PacketListener</tt> for auth &lt;iq/&gt; packets, and sends the
     * initial "jabber:iq:auth" request.</p>
     *
     * @throws IllegalStateException If the auth <tt>PacketListener</tt> could
     * not be registered, or if any of the required properties are not valid.
     * @throws StreamException If there is a stream-level error
     * @since   JSO-0.12
     */
    protected void startConsumption() throws Throwable {
        FeatureContext      ctx = getFeatureContext();
        PacketRouter        router = ctx.getRouter();
        PacketDispatcher    dispatch = ctx.getDispatcher();

        startAuth(router, dispatch);
    }

    private void startAuth(PacketRouter router, PacketDispatcher dispatch) throws IllegalStateException, StreamException {
        StreamDataFactory   sdf = router.getDataFactory();
        InfoQuery           iq;
        AuthQuery           auth;
        AuthWatcher         watch;
        String              usr = getUsername(), res = getResource();

        //Validate properties
        if (!Utilities.isValidString(usr))
            throw new IllegalStateException("username is not valid");
        if (!Utilities.isValidString(res))
            throw new IllegalStateException("resource is not valid");
        
        try {
            watch = new AuthWatcher(router);
            dispatch.addPacketListener(PacketEvent.RECEIVED, watch);
        } catch (SAXPathException spe) {
            throw new IllegalStateException("Could not register Auth packet listener");
        }
        
        iq = (InfoQuery)sdf.createPacketNode(sdf.createNSI("iq", router.getDefaultNamespace()), InfoQuery.class);
        iq.setID(IdentityGenerator.generateGlobal("auth"));
        iq.setTo(getServer());
        iq.setType(InfoQuery.GET);
        
        auth = (AuthQuery)iq.addElement(AuthQuery.NAME, AuthQuery.class);
        auth.setUsername(usr);
        auth.setResource(res);

        ((SimpleVariableContext)watch.getXPath().getVariableContext()).setVariableValue("id", iq.getID());
        router.send(iq);
    }
    /**
     * <p>
     * Called during consumption to configure the return query. This method
     * determines if either {@link AuthQuery#DIGEST} or {@link AuthQuery#PLAIN}
     * is supported (in that order), and configures the return query
     * accordingly. If neither is supported, a "not-acceptable"
     * <tt>PacketException</tt> is thrown.</p>
     *
     * <p>
     * Subclasses may override this method to handle additional authentication
     * methods.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. The returned
     * query is a copy of <tt>in</tt>, with the proper method fields set.</p>
     *
     * <p>
     * The value of <tt>r</tt> should never be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>in</tt> should never be <tt>null</tt>.</p>
     *
     * @param r The <tt>PacketRouter</tt> to use for configuration.
     * @param in The <tt>AuthQuery</tt> to use for configuration.
     * @return The configured <tt>AuthQuery</tt>.
     * @throws PacketException If configuration could not be performed.
     * @since   JSO-0.12
     */
    protected AuthQuery configureAuth(PacketRouter r, AuthQuery in) throws PacketException {
        AuthQuery       out = null;
        
        if          (AuthQuery.DIGEST.isSupported(in)) {
            out = configureDigestAuth(r, in);
        } else if   (AuthQuery.PLAIN.isSupported(in)) {
            out = configurePlainAuth(r, in);
        }
        
        if (out == null)
            throw new PacketException(PacketError.CANCEL, PacketError.NOT_ACCEPTABLE_CONDITION);
        
        return out;
    }
    private final AuthQuery configureDigestAuth(PacketRouter r, AuthQuery in) {
        AuthQuery.Method    method = AuthQuery.DIGEST;
        AuthQuery           out = (AuthQuery)in.copy();
        Map                 params = method.setupAuthParams(out);
        String              pass = getPassword();

        params.put("sessionid", r.getInboundContext().getID());
        params.put("password", pass);
        method.setupAuth(out, params);
            
        return out;
    }
    private final AuthQuery configurePlainAuth(PacketRouter r, AuthQuery in) {
        AuthQuery.Method    method = AuthQuery.PLAIN;
        AuthQuery           out = (AuthQuery)in.copy();
        Map                 params = method.setupAuthParams(out);
        String              pass = getPassword();

        params.put("password", pass);
        method.setupAuth(out, params);
        
        return out;
    }
}