/*
 * BindFeatureConsumer.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import java.util.Collections;
import java.util.Map;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.InfoQuery;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.PacketRouter;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamException;
import org.jabberstudio.jso.StreamFeature;
import org.jabberstudio.jso.StreamFeatureset;
import org.jabberstudio.jso.event.PacketDispatcher;
import org.jabberstudio.jso.event.PacketEvent;
import org.jabberstudio.jso.features.AbstractFeatureConsumer;
import org.jabberstudio.jso.features.FeatureContext;
import org.jabberstudio.jso.util.IdentityGenerator;
import org.jabberstudio.jso.util.PacketException;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.util.XPathListener;

/**
 * <p>
 * <tt>FeatureConsumer</tt> implementation for consuming resource binding.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class BindFeatureConsumer extends AbstractFeatureConsumer {
    //Inner-classes
    private class BindWatcher extends XPathListener {
        //Constructors
        public BindWatcher(PacketRouter router) throws SAXPathException {
            super(router.getDataFactory(), "app:iq[(@type='result') or (@type='error')]/bind:*");
            
            Map     nss = new java.util.HashMap();
            nss.put("app", router.getDefaultNamespace());
            nss.put("bind", BindQuery.NAMESPACE);
            
            setupNamespaces(nss);
        }
        
        //Methods
        public void packetMatched(PacketEvent evt) {
            PacketDispatcher    dispatch = getFeatureContext().getDispatcher();
            InfoQuery           iq = (InfoQuery)evt.getData();
            BindQuery           bind;

            dispatch.removePacketListener(PacketEvent.RECEIVED, this);
            if (iq.getType() == InfoQuery.ERROR) {
                completeFailedConsumption(new PacketException(iq.getError()));
            } else {
                bind = (BindQuery)iq.getFirstElement(BindQuery.class);
                setJID(bind.getJID());
                
                completeSucceededConsumption(false);
            }
        }
    }
    
    //Fields
    private JID                 _JID;
    private String              _Resource;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>BindFeatureConsumer</tt>. By default, this constructor
     * sets this <tt>BindFeatureConsumer</tt> to request a server-provided
     * resource.</p>
     *
     * @since   JSO-0.12
     */
    public BindFeatureConsumer() {
        this(null);
    }
    /**
     * <p>
     * Creates a new <tt>BindFeatureConsumer</tt>, requesting the given
     * resource.</p>
     *
     * <p>
     * The value of <tt>res</tt> must be a valid resource (including the
     * empty string and <tt>null</tt>), or an <tt>IllegalArgumentException</tt>
     * is thrown.</p>
     *
     * @param res The resource to request.
     * @since   JSO-0.12
     */
    public BindFeatureConsumer(String res) throws IllegalArgumentException {
        setResource(res);
        
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the JID this <tt>BindFeatureConsumer</tt> is bound to.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the request
     * has not completed successfully.</p>
     *
     * @return The bound JID
     * @since   JSO-0.12
     */
    public JID getJID() {
        return _JID;
    }
    private void setJID(JID jid) {
        _JID = jid;
    }
    
    /**
     * <p>
     * Retrieves the resource this <tt>BindFeatureConsumer</tt> will attempt
     * to bind. A value of "" means the server-provided resource is
     * requested.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The resource to bind.
     * @since   JSO-0.12
     */
    public String getResource() {
        return _Resource;
    }
    /**
     * <p>
     * Sets the resource this <tt>BindFeatureConsumer</tt> will attempt to
     * bind.</p>
     *
     * <p>
     * The value of <tt>res</tt> may be "" or <tt>null</tt>, to request a
     * server-provided resource. If <tt>res</tt> is not a valid resource string,
     * an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param res The new resource to bind.
     * @throws IllegalArgumentException If the parameter is not valid.
     * @since   JSO-0.12
     * @see     JID#isValidResource
     */
    public void setResource(String res) throws IllegalArgumentException {
        if (!JID.isValidResource(res))
            throw new IllegalArgumentException("String \"" + res + "\" is not a valid resource");
        _Resource = ((res != null) ? res : "");
    }
    
    /**
     * <p>
     * Finds the <tt>StreamFeature</tt> in the given <tt>StreamFeatureset</tt>
     * that can be consumed. This implementation searches for the first
     * <tt>StreamElement</tt> with the NSI of {@link BindQuery#NAME}
     * in <tt>fs</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if a consumable
     * <tt>StreamFeature</tt> could not be found.</p>
     *
     * <p>
     * The value of <tt>fs</tt> will never be <tt>null</tt>.</p>
     *
     * @param fs The <tt>StreamFeatureset</tt> to search in
     * @return The consumable <tt>StreamFeature</tt>.
     * @since   JSO-0.12
     */
    public StreamFeature findFeature(StreamFeatureset fs) {
        return (StreamFeature)fs.getFirstElement(BindQuery.NAME);
    }
    /**
     * <p>
     * Begins the resource bind process. This method registers a
     * <tt>PacketListener</tt> for bind &lt;iq/&gt; packets, and sends the
     * resource bind request.</p>
     *
     * @throws IllegalStateException If the Bind <tt>PacketListener</tt> could
     * not be registered.
     * @throws StreamException If there is a stream-level error
     * @since   JSO-0.12
     */
    protected void startConsumption() throws IllegalStateException, StreamException {
        FeatureContext      ctx = getFeatureContext();
        PacketRouter        router = ctx.getRouter();
        PacketDispatcher    dispatch = ctx.getDispatcher();
        StreamDataFactory   sdf = ctx.getRouter().getDataFactory();
        InfoQuery           iq;
        BindQuery           bind;

        try {
            dispatch.addPacketListener(PacketEvent.RECEIVED, new BindWatcher(router));
        } catch (SAXPathException spe) {
            throw new IllegalStateException("Could not register Bind packet listener");
        }
        
        iq = (InfoQuery)sdf.createPacketNode(sdf.createNSI("iq", router.getDefaultNamespace()), InfoQuery.class);
        iq.setType(InfoQuery.SET);
        iq.setID(IdentityGenerator.generateGlobal("bind"));

        bind = (BindQuery)iq.addElement(BindQuery.NAME, BindQuery.class);
        bind.setResource(getResource());

        router.send(iq);
    }
    public void reset() {
        super.reset();
        
        setJID(null);
    }
}