/*
 * PrivacyQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import java.util.List;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing a "jabber:iq:privacy" query.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface PrivacyQuery extends Extension {
    //"Constants"
    /** The namespace URI for <tt>PrivacyQuery</tt> and its known children
     * (jabber:iq:privacy).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE       = "jabber:iq:privacy";
    /** The qualified name for <tt>PrivacyQuery</tt> ({jabber:iq:privacy}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME            = new NSI("query", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Determines if this <tt>PrivacyQuery</tt> declares an active list name.
     * If this method returns <tt>true</tt>, then a &lt;active/&gt; is
     * contained.</p>
     *
     * @return <tt>true</tt> if the &lt;active/&gt; element is present.
     * @since   JSO-0.12
     */
    public boolean hasActive();
    /**
     * <p>
     * Retrieves the name of the active privacy list. This method uses the
     * name attribute from the &lt;active/&gt; element, if present.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, although it
     * may be "" if the active list has not been set.</p>
     *
     * @return The name of the active privacy list.
     * @since   JSO-0.12
     */
    public String getActive();
    /**
     * <p>
     * Sets the name of the active privacy list. Note that this method always
     * results in an &lt;active/&gt; element being applied; use {@link
     * #removeActive} to remove this element.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * the &lt;active/&gt; element is still applied, but without a name
     * attribute.</p>
     *
     * @param name The new active list name.
     * @since   JSO-0.12
     */
    public void setActive(String name);
    /**
     * <p>
     * Removes any active list declaration from this <tt>PrivacyQuery</tt>.
     * This method removes all &lt;active/&gt; elements.</p>
     *
     * @since   JSO-0.12
     */
    public void removeActive();

    /**
     * <p>
     * Determines if this <tt>PrivacyQuery</tt> declares a default list name.
     * If this method returns <tt>true</tt>, then a &lt;default/&gt; is
     * contained.</p>
     *
     * @return <tt>true</tt> if the &lt;default/&gt; element is present.
     * @since   JSO-0.12
     */
    public boolean hasDefault();
    /**
     * <p>
     * Retrieves the name of the default privacy list. This method uses the
     * name attribute from the &lt;default/&gt; element, if present.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, alhtough it
     * may be "" if the default list has not been set.</p>
     *
     * @return The name of the default privacy list.
     * @since   JSO-0.12
     */
    public String getDefault();
    /**
     * <p>
     * Sets the name of the default privacy list. Note that this method always
     * results in a &lt;default/&gt; element being applied; use {@link
     * #removeDefault} to remove this element.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * the &lt;default/&gt; element is still applied, but without a name
     * attribute.</p>
     *
     * @param name The new default list name.
     * @since   JSO-0.12
     */
    public void setDefault(String name);
    /**
     * <p>
     * Removes any default list declaration from this <tt>PrivacyQuery</tt>.
     * This method removes all &lt;default/&gt; elements.</p>
     *
     * @since   JSO-0.12
     */
    public void removeDefault();
    
    /**
     * <p>
     * Retrieves the list of privacy lists contained by this
     * <tt>PrivacyQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the contained list is a <tt>PrivacyList</tt> instance.</p>
     *
     * @return The list of contained <tt>PrivacyList</tt> instances.
     * @since   JSO-0.12
     */
    public List listPrivacyLists();
    /**
     * <p>
     * Retrieves the named privacy list from this <tt>PrivacyQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if there is no
     * <tt>PrivacyList</tt> named <tt>name</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The name of the privacy list.
     * @return The named privacy list.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public PrivacyList getPrivacyList(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Adds or retrieves the named privacy list to this
     * <tt>PrivacyQuery</tt>. If a list named <tt>name</tt> is already present,
     * this method returns that instance.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The name of the privacy list.
     * @return The named privacy list.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public PrivacyList addPrivacyList(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Removes all privacy lists from this <tt>PrivacyQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearPrivacyLists();
}