/*
 * DiscoItemsQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.disco;


import java.util.List;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing Service Discovery items queries, as
 * documented in JEP-0030: Service Discovery.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href='http://www.jabber.org/jeps/jep-0030.html'>JEP-0030</a>
 */
public interface DiscoItemsQuery extends Extension {
    //"Constants"
    /**
     * The namespace URI for <tt>DiscoItemsQuery</tt>
     * (http://jabber.org/protocol/disco#items).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE = DiscoUtilities.NAMESPACE + "#items";
    /**
     * The qualified name for <tt>DiscoItemsQuery</tt>
     * ({http://jabber.org/protocol/disco#items}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME = new NSI("query", NAMESPACE);

    //Methods
    /**
     * <p>
     * Retrieves the node for this <tt>DiscoItemsQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The node for this <tt>DiscoItemsQuery</tt>.
     * @since   JSO-0.12
     */
    public String getNode();
    /**
     * <p>
     * Sets the node for this <tt>DiscoItemsQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>node</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.</p>
     *
     * @param node The new node.
     * @since   JSO-0.12
     */
    public void setNode(String node);

    /**
     * <p>
     * Retrieves the list of <tt>DiscoItem</tt> objects for this
     * <tt>DiscoItemsQuery</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a non=<tt>null</tt> instance of
     * <tt>DiscoItem</tt>.</p>
     *
     * @return The list of <tt>DiscoItem</tt> objects.
     * @since   JSO-0.12
     */
    public List listItems();
    /**
     * <p>
     * Retrieves the <tt>DiscoItem</tt> with the given JID and no node.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>jid</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param jid The JID to find.
     * @return The <tt>DiscoItem</tt> found for the JID.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoItem getItem(JID jid) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the <tt>DiscoItem</tt> with the given JID and node.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>jid</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>node</tt> may be <tt>null</tt> or "", in which case
     * this method behaves exactly like {@link #getItem(JID)}.</p>
     *
     * @param jid The JID to find.
     * @param node The node to find.
     * @return The <tt>DiscoItem</tt> found for the JID and node.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoItem getItem(JID jid, String node) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and adds an <tt>DiscoItem</tt> with the given JID and no node. The
     * name for this <tt>DiscoItem</tt> is not set.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If an
     * <tt>DiscoItem</tt> for the given JID is already present, that instance is
     * returned unmodified.</p>
     *
     * <p>
     * The value of <tt>jid</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param jid The JID to add.
     * @return The new <tt>DiscoItem</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoItem addItem(JID jid) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and adds an <tt>DiscoItem</tt> with the given JID and node. The
     * name for this <tt>DiscoItem</tt> is not set.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If an
     * <tt>DiscoItem</tt> for the given JID and node is already present, that
     * instance is returned unmodified.</p>
     *
     * <p>
     * The value of <tt>jid</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>node</tt> may be <tt>null</tt> or "", in which case
     * this method behaves exactly like {@link #addItem(JID)}.</p>
     *
     * @param jid The JID to add.
     * @param node The node to add.
     * @return The new <tt>DiscoItem</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoItem addItem(JID jid, String node) throws IllegalArgumentException;
    /**
     * <p>
     * Creates and adds an <tt>DiscoItem</tt> with the given JID, node, and name.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If an
     * <tt>DiscoItem</tt> for the given JID and node is already present, that
     * instance is returned unmodified.</p>
     *
     * <p>
     * The value of <tt>jid</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>node</tt> may be <tt>null</tt> or "", in which case
     * this method behaves exactly like {@link #addItem(JID)}, while also
     * setting the name.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * this method behaves exactly like {@link #addItem(JID, String)}.</p>
     *
     * @param jid The JID to add.
     * @param node The node to add.
     * @param name The name to add.
     * @return The new <tt>DiscoItem</tt>.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public DiscoItem addItem(JID jid, String node, String name) throws IllegalArgumentException;
    /**
     * <p>
     * Removas the <tt>DiscoItem</tt> with the given JID and no node.</p>
     *
     * <p>
     * The value of <tt>jid</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param jid The JID to remove.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public void removeItem(JID jid) throws IllegalArgumentException;
    /**
     * <p>
     * Removes the <tt>DiscoItem</tt> with the given JID and node.</p>
     *
     * <p>
     * The value of <tt>jid</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>node</tt> may be <tt>null</tt> or "", in which case
     * this method behaves exactly like {@link #removeItem(JID)}.</p>
     *
     * @param jid The JID to remove.
     * @param node The node to remove.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public void removeItem(JID jid, String node) throws IllegalArgumentException;
    /**
     * <p>
     * Removes all <tt>DiscoItem</tt> objects from this <tt>DiscoItemsQuery</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearItems();
}