/*
 * SIQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.si;


import java.util.List;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.x.fneg.FeatureNegQuery;

/**
 * <p>
 * Defines the Stream Initiation query interface. Through this interface, all
 * other information about an SI request/response is obtained.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface SIQuery extends Extension {
    //"Constants"
    /** Namespace URI for <tt>SIQuery</tt> (http://jabber.org/protocol/si).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE   = "http://jabber.org/protocol/si";
    /** Qualified name for <tt>SIQuery</tt> ({http://jabber.org/protocol/si}si).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("si", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the MIME-type for this <tt>SIQuery</tt>. This value is the
     * expected MIME-type for the data to be transferred.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The expected MIME-type.
     * @since   JSO-0.12
     */
    public String getMimeType();
    /**
     * <p>
     * Sets the MIME-type for this <tt>SIQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>mime</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.</p>
     *
     * @param mime The new expected MIME-type.
     * @since   JSO-0.12
     */
    public void setMimeType(String mime);

    /**
     * <p>
     * Retrieves the profile information for this <tt>SIQuery</tt>. This value
     * contains additional information on the purpose of the SI request.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * previously been set.</p>
     *
     * @return The profile information.
     * @since   JSO-0.12
     */
    public SIProfile getProfile();
    /**
     * <p>
     * Sets the profile information for this <tt>SIQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>prof</tt> may be <tt>null</tt>, in which case any
     * previous profile information is removed.</p>
     *
     * @param prof The new profile information.
     * @since   JSO-0.12
     */
    public void setProfile(SIProfile prof);
    /**
     * <p>
     * Generates and sets the profile information for this <tt>SIQuery</tt>.
     * This method attempts to create a <tt>SIProfile</tt> instance using the
     * given name.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown. If <tt>name</tt> does not
     * contain a valid local-name and namespace URI, and does not match a valid
     * profile (known or unknown), an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * @return The newly created and added <tt>SIProfile</tt>.
     * @param name The name of the <tt>SIProfile</tt> to create and add.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public SIProfile applyProfile(NSI name) throws IllegalArgumentException;
    /**
     * <p>
     * Generates and sets the profile information for this <tt>SIQuery</tt>.
     * This method attepmts to create a <tt>SIProfile</tt> instance using the
     * given name and interface type.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or an
     * <tt>IllegalArgumentException</tt> is thrown. If <tt>name</tt> does not
     * contain a valid local-name and namespace URI, and does not match a valid
     * profile (known or unknown), an <tt>IllegalArgumentException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value of <tt>iface</tt> must be <tt>null</tt> or a class
     * implementing or extending <tt>SIProfile</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @return The newly created and added <tt>SIProfile</tt>.
     * @param name The name of the <tt>SIProfile</tt> to create and add.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public SIProfile applyProfile(NSI name, Class iface) throws IllegalArgumentException;
    
    /**
     * <p>
     * Retrieves the <tt>FeatureNegQuery</tt> for this <tt>SIQuery</tt>. This
     * value is used to negotiate the stream method.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If it is not
     * present, a <tt>FeatureNegQuery</tt> is first created.</p>
     *
     * @return The <tt>FeatureNegQuery</tt> for stream method.
     * @since   JSO-0.12
     */
    public FeatureNegQuery getStreamMethodNegotiation();

    /**
     * <p>
     * Retrieves the list of supported stream methods for this <tt>SIQuery</tt>.
     * This method is a convenience over {@link
     * FeatureNegQuery#getSupportedOptions} specifically for the
     * "stream-method" feature.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is the string name for the method.</p>
     *
     * @return The list of supported stream methods.
     * @since   JSO-0.12
     */
    public List listSupportedStreamMethods();
    /**
     * <p>
     * Adds the given stream method to the list of those supported by this
     * <tt>SIQuery</tt>. This method is a convenience over {@link
     * FeatureNegQuery#putFeature}, using "stream-method" as the feature var,
     * and the existing list (with <tt>method</tt> added) for the options.</p>
     *
     * <p>
     * The value of <tt>method</tt> may be <tt>null</tt> or "", in which case
     * no actions are taken. If <tt>method</tt> is already in the list, no
     * actions are taken.</p>
     *
     * @param method The stream method to add.
     * @since   JSO-0.12
     */
    public void addSupportedStreamMethod(String method);
    /**
     * <p>
     * Removes the given stream method from the list of those supported by this
     * <tt>SIQuery</tt>. This method is a convenience over {@link
     * FeatureNegQuery#putFeature}, using "stream-method" as the feature var,
     * and the existing list (with <tt>method</tt> removed) for the options.</p>
     *
     * <p>
     * The value of <tt>method</tt> may be <tt>null</tt> or "", in which case
     * no actions are taken. If <tt>method</tt> is not in the list, no
     * actions are taken.</p>
     *
     * @param method The stream method to remove.
     * @since   JSO-0.12
     */
    public void removeSupportedStreamMethod(String method);
    /**
     * <p>
     * Clears all supported stream methods from this <tt>SIQuery</tt>. This
     * method is a convenience over {@link FeatureNegQuery#putFeature}, using
     * "stream-method" as the feature var and an empty list for the options.</p>
     *
     * @since   JSO-0.12
     */
    public void clearSupportedStreamMethods();

    /**
     * <p>
     * Retrieves the preferred stream method for this <tt>SIQuery</tt>. This
     * method is a convenience over {@link
     * FeatureNegQuery#getPreferredOption}, using "stream-method" for the
     * feature var.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return the preferred stream method.
     * @since   JSO-0.12
     */
    public String getPreferredStreamMethod();
    /**
     * <p>
     * Sets the preferred stream method for this <tt>SIQuery</tt>. This method
     * is a convenience over {@link FeatureNegQuery#setPreferredOption}, using
     * "stream-method" for the feature var and <tt>method</tt> for the
     * value.</p>
     *
     * <p>
     * The value of <tt>method</tt> may be <tt>null</tt> or "", in which case
     * any previous value is removed.</p>
     *
     * @param method the new stream method.
     * @since   JSO-0.12
     */
    public void setPreferredStreamMethod(String method);
}